import json

none = "d3043820717d74d9a17694c176d39733"

# region Subscription


class Subscription:
    """
    # Arguments
    resource_id: str
    protocol: str
    endpoint: str
    event_type: str
    event_format: dict
    """

    def __init__(
            self,
            resource_id=none,
            protocol=none,
            endpoint=none,
            event_type=none,
            event_format=none):

        self.resource_id = resource_id
        self.protocol = protocol
        self.endpoint = endpoint
        self.event_type = event_type
        self.event_format = event_format

# endregion


class SubscriptionRequest:
    def __init__(self, subscription):
        self.subscription = subscription

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)



import json

none = "d3043820717d74d9a17694c176d39733"


# region Elastigroup
class Elastigroup:
    """
    # Arguments
    name: str
    region: str
    resource_group_name: str
    capacity: Capacity
    strategy: Strategy
    compute: Compute
    scaling: Scaling
    scheduling: Scheduling
    third_parties_integration: ThirdPartiesIntegration
    """

    def __init__(
            self,
            name=none,
            region=none,
            resource_group_name=none,
            capacity=none,
            strategy=none,
            compute=none,
            scaling=none,
            scheduling=none,
            third_parties_integration=none):

        self.name = name
        self.region = region
        self.resource_group_name = resource_group_name
        self.capacity = capacity
        self.strategy = strategy
        self.compute = compute
        self.scaling = scaling
        self.scheduling = scheduling
        self.third_parties_integration = third_parties_integration
# endregion

# region Capacity


class Capacity:
    """
    # Arguments
    minimum: int
    maximum: int
    target: int
    """

    def __init__(
            self,
            minimum=none,
            maximum=none,
            target=none):

        self.minimum = minimum
        self.maximum = maximum
        self.target = target
# endregion


# region Strategy
class Strategy:
    """
    # Arugments
    low_priority_percentage: int
    on_demand_count: int
    draining_timeout: int
    """

    def __init__(
            self,
            low_priority_percentage=none,
            on_demand_count=none,
            draining_timeout=none):

        self.low_priority_percentage = low_priority_percentage
        self.on_demand_count = on_demand_count
        self.draining_timeout = draining_timeout
# endregion

# region Compute


class Compute:
    """
    # Arguments
    vm_sizes: VmSizes
    product: str
    health: Health
    launch_specification: LaunchSpecification
    """

    def __init__(
            self,
            vm_sizes=none,
            product=none,
            health=none,
            launch_specification=none):

        self.vm_sizes = vm_sizes
        self.product = product
        self.health = health
        self.launch_specification = launch_specification


class VmSizes:
    """
    #Arguments
    od_sizes: list[str]
    low_priority_sizes: list[str]
    """

    def __init__(
            self,
            od_sizes=none,
            low_priority_sizes=none):

        self.od_sizes = od_sizes
        self.low_priority_sizes = low_priority_sizes


class Health:
    """
    #Arguments
    health_check_type: str
    auto_healing: bool
    grace_period: int
    """

    def __init__(
            self,
            health_check_type=none,
            auto_healing=none,
            grace_period=none):

        self.health_check_type = health_check_type
        self.auto_healing = auto_healing
        self.grace_period = grace_period


class LaunchSpecification:
    """
    #Arguments
    image: Image
    network: Network
    login: Login
    user_data: str
    shutdown_script: str
    custom_data: str
    load_balancers_config: LoadBalancerConfig
    tags: list[Tag]
    extensions: list[Extension]
    managed_service_identities: list[ManagedServiceIdentity]
    """

    def __init__(
            self,
            image=none,
            network=none,
            login=none,
            user_data=none,
            shutdown_script=none,
            custom_data=none,
            load_balancers_config=none,
            tags=none,
            extensions=none,
            managed_service_identities=none):

        self.image = image
        self.network = network
        self.login = login
        self.user_data = user_data
        self.shutdown_script = shutdown_script
        self.custom_data = custom_data
        self.load_balancers_config = load_balancers_config
        self.tags = tags
        self.extensions = extensions
        self.managed_service_identities = managed_service_identities


class Image:
    """
    # Arguments
    marketplace: Marketplace
    custom: Custom
    """

    def __init__(self, marketplace=none, custom=none):

        self.marketplace = marketplace
        self.custom = custom


class Marketplace:
    """
    # Arguments
    publisher: str
    offer: str
    sku: str
    version: str
    """

    def __init__(
            self,
            publisher=none,
            offer=none,
            sku=none,
            version=none):

        self.publisher = publisher
        self.offer = offer
        self.sku = sku
        self.version = version


class Custom:
    """
    # Arguments
    resource_group_name: str
    image_name: str
    """

    def __init__(
            self,
            resource_group_name=none,
            image_name=none):

        self.resource_group_name = resource_group_name
        self.image_name = image_name


class Network:
    """
    # Arguments
    virtual_network_name: str
    subnet_name: str
    resource_group_name: str
    assign_public_ip: bool
    additional_ip_configurations: list[AdditionalIpConfiguration]
    """

    def __init__(
            self,
            virtual_network_name=none,
            subnet_name=none,
            resource_group_name=none,
            assign_public_ip=none,
            additional_ip_configurations=none):

        self.virtual_network_name = virtual_network_name
        self.subnet_name = subnet_name
        self.resource_group_name = resource_group_name
        self.assign_public_ip = assign_public_ip
        self.additional_ip_configurations = additional_ip_configurations


class AdditionalIpConfiguration:
    """
    # Arguments
    name: str
    private_ip_address_version: str
    """

    def __init__(
            self,
            name=none,
            private_ip_address_version=none):

        self.name = name
        self.private_ip_address_version = private_ip_address_version


class Login:
    """
    # Arguments
    ssh_public_key: str
    user_name: str
    password: str
    """

    def __init__(
            self,
            ssh_public_key=none,
            user_name=none,
            password=none):

        self.ssh_public_key = ssh_public_key
        self.user_name = user_name
        self.password = password


class LoadBalancerConfig:
    """
    # Arguments
    load_balancers: list[LoadBalancer]
    """

    def __init__(self, load_balancers=none):
        self.load_balancers = load_balancers


class LoadBalancer:
    """
    # Arguments
    balancer_id: str
    target_set_id: str
    auto_weight: bool
    resource_group_name: str
    application_gateway_name: str
    backend_pool_name: str
    balancer_type: str
    """

    def __init__(
            self,
            balancer_id=none,
            target_set_id=none,
            auto_weight=none,
            resource_group_name=none,
            application_gateway_name=none,
            backend_pool_name=none,
            balancer_type=none):

        self.balancer_id = balancer_id
        self.target_set_id = target_set_id
        self.auto_weight = auto_weight
        self.resource_group_name = resource_group_name
        self.application_gateway_name = application_gateway_name
        self.backend_pool_name = backend_pool_name
        self.type = balancer_type


class Tag:
    """
    # Arguments
    tag_key: str
    tag_value: str
    """

    def __init__(
            self,
            tag_key=none,
            tag_value=none):

        self.tag_key = tag_key
        self.tag_value = tag_value


class Extension:
    """
    # Arguments
    auto_upgrade_minor_version: bool
    name: str
    publisher: str
    extension_type: str
    type_handler_version: str
    protected_settings: ProtectedSettings
    """

    def __init__(
            self,
            auto_upgrade_minor_version=none,
            name=none,
            publisher=none,
            extension_type=none,
            type_handler_version=none,
            protected_settings=none):

        self.auto_upgrade_minor_version = auto_upgrade_minor_version
        self.name = name
        self.publisher = publisher
        self.type = extension_type
        self.type_handler_version = type_handler_version
        self.protected_settings = protected_settings


class ProtectedSettings:
    """
    # Arguments
    command_to_execute: str
    """

    def __init__(self, command_to_execute=none):
        self.command_to_execute = command_to_execute


class ManagedServiceIdentity:
    """
    # Arguments
    resource_group_name: str
    name: str
    """

    def __init__(
            self,
            resource_group_name=none,
            name=none):

        self.resource_group_name = resource_group_name
        self.name = name

# endregion


# region Scaling
class Scaling:
    """
    # Arguments
    up:  list[ScalingPolicy]
    down: list[ScalingPolicy]
    """

    def __init__(self, up=none, down=none):

        self.up = up
        self.down = down


class ScalingPolicy:
    """
    # Arguments
    policy_name: str
    namespace: str
    metric_name: str
    dimensions: list[ScalingPolicyDimension]
    statistic: str
    unit: str
    threshold: float
    adjustment: int
    min_target_capacity: int
    period: int
    evaluation_periods: int
    cooldown: int
    action: ScalingPolicyAction
    operator: str
    """

    def __init__(
            self,
            policy_name=none,
            namespace=none,
            metric_name=none,
            dimensions=none,
            statistic=none,
            unit=none,
            threshold=none,
            adjustment=none,
            min_target_capacity=none,
            period=none,
            evaluation_periods=none,
            cooldown=none,
            action=none,
            operator=none):

        self.policy_name = policy_name
        self.namespace = namespace
        self.metric_name = metric_name
        self.dimensions = dimensions
        self.statistic = statistic
        self.unit = unit
        self.threshold = threshold
        self.adjustment = adjustment
        self.min_target_capacity = min_target_capacity
        self.period = period
        self.evaluation_periods = evaluation_periods
        self.cooldown = cooldown
        self.action = action
        self.operator = operator


class ScalingPolicyDimension:
    """
    # Arguments
    name: str
    value: str
    """

    def __init__(self, name=none, value=none):

        self.name = name
        self.value = value


class ScalingPolicyAction:
    """
    # Arguments
    scaling_type: str
    adjustment: int
    min_target_capacity: int
    target: int
    maximum: int
    minimum: int
    """

    def __init__(
            self,
            scaling_type=none,
            adjustment=none,
            min_target_capacity=none,
            target=none,
            maximum=none,
            minimum=none):

        self.type = scaling_type
        self.adjustment = adjustment
        self.min_target_capacity = min_target_capacity
        self.target = target
        self.maximum = maximum
        self.minimum = minimum
# endregion


# region Scheduling
class Scheduling:
    """
    # Arguments
    tasks: list[SchedulingTask]
    """

    def __init__(self, tasks=none):
        self.tasks = tasks


class SchedulingTask:
    """
    # Arguments
    is_enabled: bool
    cron_expression: str
    task_type: str
    scale_target_capacity: int
    scale_min_capacity: int
    scale_max_capacity: int
    batch_size_percentage: int
    grace_period: int
    adjustment: int
    adjustment_percentage: int
    """

    def __init__(
            self,
            is_enabled=none,
            cron_expression=none,
            task_type=none,
            scale_target_capacity=none,
            scale_min_capacity=none,
            scale_max_capacity=none,
            batch_size_percentage=none,
            grace_period=none,
            adjustment=none,
            adjustment_percentage=none):

        self.is_enabled = is_enabled
        self.cron_expression = cron_expression
        self.task_type = task_type
        self.scale_target_capacity = scale_target_capacity
        self.scale_min_capacity = scale_min_capacity
        self.scale_max_capacity = scale_max_capacity
        self.batch_size_percentage = batch_size_percentage
        self.grace_period = grace_period
        self.adjustment = adjustment
        self.adjustment_percentage = adjustment_percentage

# endregion


# region Third Parties Integration
class ThirdPartiesIntegration:
    """
    # Arguments
    kubernetes: Kubernetes
    hpc_grid_engine: HpcGridEngine
    """

    def __init__(self, kubernetes=none, hpc_grid_engine=none):
        self.kubernetes = kubernetes
        self.hpc_grid_engine = hpc_grid_engine


class Kubernetes:
    """
    # Arguments
    cluster_identifier: str
    """

    def __init__(self, cluster_identifier=none):
        self.cluster_identifier = cluster_identifier


class HpcGridEngine:
    """
    # Arguments
    cluster_id: str
    queues: List[str]
    """

    def __init__(self, cluster_id=none, queues=none):
        self.cluster_id = cluster_id
        self.queues = queues


class ElastigroupCreationRequest:
    def __init__(self, elastigroup):
        self.group = elastigroup

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion


# Deployment
class RollGroup:
    """
    # Arguments
    batch_size_percentage: int
    grace_period: int
    health_check_type: str
    """

    def __init__(
            self,
            batch_size_percentage=none,
            grace_period=none,
            health_check_type=none):

        self.batch_size_percentage = batch_size_percentage
        self.grace_period = grace_period
        self.health_check_type = health_check_type


class ElastigroupRollRequest:
    def __init__(self, group_roll):
        self.batch_size_percentage = group_roll.batch_size_percentage
        self.grace_period = group_roll.grace_period
        self.health_check_type = group_roll.health_check_type

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion


# Detach Instances
class DetachConfiguration:
    """
    # Arguments
    instances_to_detach: list[str]
    draining_timeout: int
    should_decrement_target_capacity: bool
    """

    def __init__(
            self,
            instances_to_detach=none,
            draining_timeout=none,
            should_decrement_target_capacity=none):
        self.instances_to_detach = instances_to_detach
        self.draining_timeout = draining_timeout
        self.should_decrement_target_capacity = should_decrement_target_capacity


class ElastigroupDetachInstancesRequest:
    def __init__(self, detach_configuration):
        self.should_decrement_target_capacity = detach_configuration.should_decrement_target_capacity
        self.draining_timeout = detach_configuration.draining_timeout
        self.instances_to_detach = detach_configuration.instances_to_detach

    def toJSON(self):
        return json.dumps(
            self,
            default=lambda o: o.__dict__,
            sort_keys=True,
            indent=4)
# endregion



import json
from typing import List

none = "d3043820717d74d9a17694c176d39733"


# region Cluster

class Cluster:
    """
    # Arguments
    visible_to_all_users: Boolean
    termination_protected: Boolean
    keep_job_flow_alive_when_no_steps: Boolean
    log_uri: str
    additional_info: str
    job_flow_role: str
    service_role: str
    security_configuration: str
    """

    def __init__(
            self,
            visible_to_all_users: bool = none,
            termination_protected: bool = none,
            keep_job_flow_alive_when_no_steps: bool = none,
            log_uri: str = none,
            additional_info: str = none,
            job_flow_role: str = none,
            service_role: str = none,
            security_configuration: str = none):

        self.visible_to_all_users = visible_to_all_users
        self.termination_protected = termination_protected
        self.keep_job_flow_alive_when_no_steps = keep_job_flow_alive_when_no_steps
        self.log_uri = log_uri
        self.additional_info = additional_info
        self.job_flow_role = job_flow_role
        self.service_role = service_role
        self.security_configuration = security_configuration
# endregion


# region Strategy

class Wrapping:
    """
    # Arguments
    source_cluster_id: str
    """

    def __init__(
            self,
            source_cluster_id: str = none):

        self.source_cluster_id = source_cluster_id


class Cloning:
    """
    # Arguments
    origin_cluster_id: str
    include_steps: bool
    number_of_retries: int
    """

    def __init__(
            self,
            origin_cluster_id: str = none,
            include_steps: bool = none,
            number_of_retries: int = none):

        self.origin_cluster_id = origin_cluster_id
        self.include_steps = include_steps
        self.number_of_retries = number_of_retries


class New:
    """
    # Arguments
    release_label: str
    number_of_retries: int
    """

    def __init__(
            self,
            release_label: str = none,
            number_of_retries: int = none):

        self.release_label = release_label
        self.number_of_retries = number_of_retries


class ProvisioningTimeout:
    """
    # Arguments
    timeout: int
    timeout_action: str
    """

    def __init__(
            self,
            timeout: int = none,
            timeout_action: str = none):

        self.timeout = timeout
        self.timeout_action = timeout_action


class Strategy:
    """
    # Arguments
    wrapping: Wrapping
    cloning: Cloning
    new: New
    provisioning_timeout: ProvisioningTimeout
    """

    def __init__(
            self,
            wrapping: Wrapping = none,
            cloning: Cloning = none,
            new: New = none,
            provisioning_timeout: ProvisioningTimeout = none):

        self.wrapping = wrapping
        self.cloning = cloning
        self.new = new
        self.provisioning_timeout = provisioning_timeout


# endregion


# region Compute

class Application:
    """
    # Arguments
    name: str
    args: List[str]
    version: str
    """

    def __init__(
            self,
            name: str = none,
            args: List[str] = none,
            version: str = none):

        self.name = name
        self.args = args
        self.version = version


class AvailabilityZone:
    """
    # Arguments
    name: str
    subnetId: str
    """

    def __init__(
            self,
            name: str = none,
            subnet_id: str = none):

        self.name = name
        self.subnet_id = subnet_id


class File:
    """
    # Arguments
    bucket: str
    key: str
    """

    def __init__(
            self,
            bucket: str = none,
            key: str = none):

        self.bucket = bucket
        self. key = key


class ScriptBootstrapAction:
    """
    # Arguments
    args: List[str]
    path: str
    """

    def __init__(
            self,
            args: List[str] = none,
            path: str = none):

        self.args = args
        self.path = path


class JsonConfiguration:
    """
    # Arguments
    name: str
    script_bootstrap_action: ScriptBootstrapAction
    """

    def __init__(
            self,
            name: str = none,
            script_bootstrap_action: ScriptBootstrapAction = none):

        self.name = name
        self.script_bootstrap_action = script_bootstrap_action


class BootstrapActions:
    """
    # Arguments
    file: File
    json_configuration: List[JsonConfiguration]
    """

    def __init__(
            self,
            file: File = none,
            json_configuration: List[JsonConfiguration] = none):

        self.file = file
        self.json_configuration = json_configuration


class Configurations:
    """
    # Arguments
    file: File
    """

    def __init__(
            self,
            file: File = none):

        self.file = file


class DynamicVolumeSize:
    """
    # Arguments
    base_size: int
    resource: str
    size_per_resource_unit: int
    """

    def __init__(
            self,
            base_size: int = none,
            resource: str = none,
            size_per_resource_unit: int = none):

        self.base_size = base_size
        self.resource = resource
        self.size_per_resource_unit = size_per_resource_unit


class VolumeSpecification:
    """
    # Arguments
    volume_type: str
    size_in_g_b: int
    iops: int
    dynamic_volume_size: DynamicVolumeSize
    """

    def __init__(
            self,
            volume_type: str = none,
            size_in_g_b: int = none,
            iops: int = none,
            dynamic_volume_size: DynamicVolumeSize = none):

        self.volume_type = volume_type
        self.size_in_g_b = size_in_g_b
        self.iops = iops
        self.dynamic_volume_size = dynamic_volume_size


class SingleEbsConfig:
    """
    # Arguments
    volume_specification: VolumeSpecification
    volumes_per_instance: int
    """

    def __init__(
            self,
            volume_specification: VolumeSpecification = none,
            volumes_per_instance: int = none):

        self.volume_specification = volume_specification
        self.volumes_per_instance = volumes_per_instance


class EbsConfiguration:
    """
    # Arguments
    ebs_optimized: bool
    ebs_block_device_configs: List[SingleEbsConfig]
    """

    def __init__(
            self,
            ebs_optimized: bool = none,
            ebs_block_device_configs: List[SingleEbsConfig] = none):

        self.ebs_optimized = ebs_optimized
        self.ebs_block_device_configs = ebs_block_device_configs


class Capacity:
    """
    # Arguments
    target: int
    minimum: int
    maximum: int
    unit: str
    """

    def __init__(
            self,
            target: int = none,
            minimum: int = none,
            maximum: int = none,
            unit: str = none):

        self.target = target
        self.minimum = minimum
        self.maximum = maximum
        self.unit = unit


class MasterGroup:
    """
    # Arguments
    instance_types: List[str]
    target: int
    life_cycle: str
    ebs_configuration: EbsConfiguration
    configurations: Configurations
    """

    def __init__(
            self,
            instance_types: List[str] = none,
            target: int = none,
            life_cycle: str = none,
            ebs_configuration: EbsConfiguration = none,
            configurations: Configurations = none):

        self.instance_types = instance_types
        self.target = target
        self.life_cycle = life_cycle
        self.ebs_configuration = ebs_configuration
        self.configurations = configurations


class CoreGroup:
    """
    # Arguments
    instance_types: List[str]
    target: int
    capacity: Capacity
    life_cycle: str
    ebs_configuration: EbsConfiguration
    configurations: Configurations
    """

    def __init__(
            self,
            instance_types: List[str] = none,
            target: int = none,
            capacity: Capacity = none,
            life_cycle: str = none,
            ebs_configuration: EbsConfiguration = none,
            configurations: Configurations = none):

        self.instance_types = instance_types
        self.target = target
        self.capacity = capacity
        self.life_cycle = life_cycle
        self.ebs_configuration = ebs_configuration
        self.configurations = configurations


class TaskGroup:
    """
    # Arguments
    instance_types: List[str]
    capacity: Capacity
    life_cycle: str
    ebs_configuration: EbsConfiguration
    configurations: Configurations
    """

    def __init__(
            self,
            instance_types: List[str] = none,
            capacity: Capacity = none,
            life_cycle: str = none,
            ebs_configuration: EbsConfiguration = none,
            configurations: Configurations = none):

        self.instance_types = instance_types
        self.capacity = capacity
        self.life_cycle = life_cycle
        self.ebs_configuration = ebs_configuration
        self.configurations = configurations


class InstanceGroups:
    """
    # Arguments
    master_group: MasterGroup
    core_group: CoreGroup
    task_group: TaskGroup
    """

    def __init__(
            self,
            master_group: MasterGroup = none,
            core_group: CoreGroup = none,
            task_group: TaskGroup = none):

        self.master_group = master_group
        self.core_group = core_group
        self.task_group = task_group


class InstanceWeight:
    """
    # Arguments
    instance_type: str
    weighted_capacity: int
    """

    def __init__(
            self,
            instance_type: str = none,
            weighted_capacity: int = none):

        self.instance_type = instance_type
        self.weighted_capacity = weighted_capacity


class Steps:
    """
    # Arguments
    file: File
    """

    def __init__(
            self,
            file=none):

        self.file = file


class Tag:
    """
    # Arguments
    tag_key: str
    tag_value: str
    """

    def __init__(
            self,
            tag_key: str = none,
            tag_value: str = none):

        self.tag_key = tag_key
        self.tag_value = tag_value


class Compute:
    """
    # Arguments
    additional_master_security_groups: List[str]
    additional_slave_security_groups: List[str]
    applications: List[Application]
    availability_zones: List[AvailabilityZone]
    bootstrap_actions: BootstrapActions
    configurations: Configurations
    custom_ami_id: str
    ebs_root_volume_size: int
    ec2_key_name: str
    emr_managed_master_security_group: str
    emr_managed_slave_security_group: str
    instance_groups: InstanceGroups
    instance_weights: List[InstanceWeight]
    repo_upgrade_on_boot: str
    service_access_security_group: str
    steps: Steps
    tags: List[Tag]
    """

    def __init__(
            self,
            additional_master_security_groups: List[str] = none,
            additional_slave_security_groups: List[str] = none,
            applications: List[Application] = none,
            availability_zones: List[AvailabilityZone] = none,
            bootstrap_actions: BootstrapActions = none,
            configurations: Configurations = none,
            custom_ami_id: str = none,
            ebs_root_volume_size: int = none,
            ec2_key_name: str = none,
            emr_managed_master_security_group: str = none,
            emr_managed_slave_security_group: str = none,
            instance_groups: InstanceGroups = none,
            instance_weights: List[InstanceWeight] = none,
            repo_upgrade_on_boot: str = none,
            service_access_security_group: str = none,
            steps: Steps = none,
            tags: List[Tag] = none):

        self.additional_master_security_groups = additional_master_security_groups
        self.additional_slave_security_groups = additional_slave_security_groups
        self.applications = applications
        self.availability_zones = availability_zones
        self.bootstrap_actions = bootstrap_actions
        self.configurations = configurations
        self.custom_ami_id = custom_ami_id
        self.ebs_root_volume_size = ebs_root_volume_size
        self.ec2_key_name = ec2_key_name
        self.emr_managed_master_security_group = emr_managed_master_security_group
        self.emr_managed_slave_security_group = emr_managed_slave_security_group
        self.instance_groups = instance_groups
        self.instance_weights = instance_weights
        self.repo_upgrade_on_boot = repo_upgrade_on_boot
        self.service_access_security_group = service_access_security_group
        self.steps = steps
        self.tags = tags

# endregion


# region Scheduling

class Task:
    """
    # Arguments
    is_enabled: bool
    instance_group_type: str
    task_type: str
    cron_expression: str
    target_capacity: int
    min_capacity: int
    max_capacity: int
    """

    def __init__(
            self,
            is_enabled: bool = none,
            instance_group_type: str = none,
            task_type: str = none,
            cron_expression: str = none,
            target_capacity: int = none,
            min_capacity: int = none,
            max_capacity: int = none):

        self.is_enabled = is_enabled
        self.instance_group_type = instance_group_type
        self.task_type = task_type
        self.cron_expression = cron_expression
        self.target_capacity = target_capacity
        self.min_capacity = min_capacity
        self.max_capacity = max_capacity


class Scheduling:
    """
    # Arguments 
    tasks: List[Task]
    """

    def __init__(
            self,
            tasks: List[Task] = none):

        self.tasks = tasks

# endregion


# region Scaling

class Action:
    """
    # Arguments
    type: str
    adjustment: int
    min_target_capacity: int
    target: int
    minimum: int
    maximum: int
    """

    def __init__(
            self,
            type: str = none,
            adjustment: int = none,
            min_target_capacity: int = none,
            target: int = none,
            minimum: int = none,
            maximum: int = none):

        self.type = type
        self.adjustment = adjustment
        self.min_target_capacity = min_target_capacity
        self.target = target
        self.minimum = minimum
        self.maximum = maximum


class Dimension:
    """
    # Arguments
    name: str
    value: str
    """

    def __init__(
            self,
            name: str = none,
            value: str = none):

        self.name = name
        self.value = value


class Metric:
    """
    # Arguments
    policy_name: str
    metric_name: str
    statistic: str
    unit: str
    threshold: int
    adjustment: int
    namespace: str
    min_target_capacity: int
    period: int
    evaluation_periods: int
    action: Action
    cooldown: int
    dimensions: List[Dimension]
    operator: str
    """

    def __init__(
            self,
            policy_name: str = none,
            metric_name: str = none,
            statistic: str = none,
            unit: str = none,
            threshold: int = none,
            adjustment: int = none,
            namespace: str = none,
            min_target_capacity: int = none,
            period: int = none,
            evaluation_periods: int = none,
            action: Action = none,
            cooldown: int = none,
            dimensions: List[Dimension] = none,
            operator: str = none):

        self.policy_name = policy_name
        self.metric_name = metric_name
        self.statistic = statistic
        self.unit = unit
        self.threshold = threshold
        self.adjustment = adjustment
        self.namespace = namespace
        self.min_target_capacity = min_target_capacity
        self.period = period
        self.evaluation_periods = evaluation_periods
        self.action = action
        self.cooldown = cooldown
        self.dimensions = dimensions
        self.operator = operator


class Scaling:
    """
    # Arguments
    up: List[Metric]
    down: List[Metric]
    """

    def __init__(
            self,
            up: List[Metric] = none,
            down: List[Metric] = none):

        self.up = up
        self.down = down

# endregion

# region TerminationPolicy


class Statement:
    """
    # Arguments
    metric_name: str
    statistic: str
    unit: str
    threshold: int
    namespace: str
    period: int
    evaluation_periods: int
    operator: str
    """

    def __init__(
            self,
            metric_name: str = none,
            statistic: str = none,
            unit: str = none,
            threshold: int = none,
            evaluation_periods: int = none,
            namespace: str = none,
            period: int = none,
            operator: str = none):

        self.metric_name = metric_name
        self.statistic = statistic
        self.unit = unit
        self.threshold = threshold
        self.evaluation_periods = evaluation_periods
        self.namespace = namespace
        self.period = period
        self.operator = operator


class TerminationPolicy:
    """
    # Arguments
    statements: List[Statement]
    """

    def __init__(
            self,
            statements: List[Statement] = none):

        self.statements = statements

# endregion

# region EMR


class EMR:
    """
    # Arguments
    name: str
    decription: str
    region: str
    strategy: Strategy
    compute: Compute
    cluster: Cluster
    scheduling: Scheduling
    scaling: Scaling
    termination_policies: List[TerminationPolicy]
    """

    def __init__(
            self,
            name: str = none,
            description: str = none,
            region: str = none,
            strategy: Strategy = none,
            compute: Compute = none,
            cluster: Cluster = none,
            scheduling: Scheduling = none,
            scaling: Scaling = none,
            termination_policies: List[TerminationPolicy] = none):

        self.name = name
        self.description = description
        self.region = region
        self.strategy = strategy
        self.compute = compute
        self.cluster = cluster
        self.scheduling = scheduling
        self.scaling = scaling
        self.termination_policies = termination_policies

# endregion


class EMRCreationRequest:
    def __init__(self, mrScaler):
        self.mrScaler = mrScaler

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)



import json
from enum import Enum
from typing import List

none = "d3043820717d74d9a17694c176d39733"


# region Persistence


class PersistenceMode(Enum):
    reattach = "reattach"
    on_launch = "onLaunch"


class Persistence:
    """
    # Arguments
    data_disks_persistence_mode: PersistenceMode
    os_disk_persistence_mode: PersistenceMode
    should_persist_data_disks: bool
    should_persist_network: bool
    should_persist_os_disk: bool
    """

    def __init__(self, data_disks_persistence_mode: PersistenceMode = none, os_disk_persistence_mode: PersistenceMode = none,
                 should_persist_data_disks: bool = none, should_persist_network: bool = none,
                 should_persist_os_disk: bool = none):
        self.data_disks_persistence_mode = data_disks_persistence_mode
        self.os_disk_persistence_mode = os_disk_persistence_mode
        self.should_persist_data_disks = should_persist_data_disks
        self.should_persist_network = should_persist_network
        self.should_persist_os_disk = should_persist_os_disk


# endregion


# region Health
class HealthCheckTypes(Enum):
    vm_state = "vmState"
    application_gateway = "applicationGateway"


class Health:
    """
    # Arguments
    health_check_types: List[HealthCheckTypes]
    auto_healing: bool
    grace_period: int
    unhealthy_duration: int
    """

    def __init__(
            self,
            health_check_types: List[HealthCheckTypes] = none,
            auto_healing: bool = none,
            grace_period: int = none,
            unhealthy_duration: int = none):
        self.health_check_types = health_check_types
        self.auto_healing = auto_healing
        self.grace_period = grace_period
        self.unhealthy_duration = unhealthy_duration


# endregion


# region Scheduling

class SchedulingTaskType(Enum):
    pause = "pause"
    resume = "resume"
    recycle = "recycle"


class SchedulingTask:
    """
    # Arguments
    is_enabled: bool
    cron_expression: str
    type: SchedulingTaskType
    """

    def __init__(
            self,
            is_enabled: bool = none,
            cron_expression: str = none,
            type: SchedulingTaskType = none):
        self.is_enabled = is_enabled
        self.cron_expression = cron_expression
        self.type = type


class Scheduling:
    """
    # Arguments
    tasks: List[SchedulingTask]
    """

    def __init__(self, tasks: List[SchedulingTask] = none):
        self.tasks = tasks


# endregion


# region Strategy

class PerformAt(Enum):
    time_window = "timeWindow"
    never = "never"
    always = "always"


class RevertToSpot:
    """
    # Arguments
    perform_at: PerformAt
    """

    def __init__(self, perform_at: PerformAt = none):
        self.perform_at = perform_at


class SignalType(Enum):
    vm_ready = "vmReady"
    vm_ready_to_shutdown = "vmReadyToShutdown"


class Signal:
    """
    # Arguments
    timeout: int
    type: SignalType
    """

    def __init__(self, timeout: int = none, type: SignalType = none):
        self.timeout = timeout
        self.type = type


class CapacityReservationGroups:
    """
    # Arguments
    name: str
    resource_group_name: str
    should_prioritize: bool
    """

    def __init__(self,
                 name: str = none,
                 resource_group_name: str = none,
                 should_prioritize: bool = none):
        self.name = name
        self.resource_group_name = resource_group_name
        self.should_prioritize = should_prioritize


class UtilizationStrategy(Enum):
    utilize_over_spot = "utilizeOverSpot"
    utilize_over_od = "utilizeOverOD"


class CapacityReservation:
    """
    # Arguments
    capacity_reservation_groups: List[CapacityReservationGroups]
    should_utilize: bool
    utilization_strategy: UtilizationStrategy
    """

    def __init__(
            self,
            capacity_reservation_groups: List[CapacityReservationGroups] = none,
            should_utilize: bool = none,
            utilization_strategy: UtilizationStrategy = none):
        self.capacity_reservation_groups = capacity_reservation_groups
        self.should_utilize = should_utilize
        self.utilization_strategy = utilization_strategy


class PreferredLifeCycle(Enum):
    spot = "spot"
    od = "od"


class Strategy:
    """
    # Arguments
    availability_vs_cost: int
    capacity_reservation: CapacityReservation
    draining_timeout: int
    fallback_to_od: bool
    od_windows: List[str]
    optimization_windows: List[str]
    preferred_lifecycle: PreferredLifeCycle
    revert_to_spot: RevertToSpot
    signals: List[Signal]
    """

    def __init__(
            self,
            availability_vs_cost: int = none,
            capacity_reservation: CapacityReservation = none,
            draining_timeout: int = none,
            fallback_to_od: bool = none,
            od_windows: List[str] = none,
            optimization_windows: List[str] = none,
            preferred_lifecycle: PreferredLifeCycle = none,
            revert_to_spot: RevertToSpot = none,
            signals: List[Signal] = none):
        self.availability_vs_cost = availability_vs_cost
        self.capacity_reservation = capacity_reservation
        self.draining_timeout = draining_timeout
        self.fallback_to_od = fallback_to_od
        self.od_windows = od_windows
        self.optimization_windows = optimization_windows
        self.preferred_lifecycle = preferred_lifecycle
        self.revert_to_spot = revert_to_spot
        self.signals = signals


# endregion


# region Compute
class StorageType(Enum):
    managed = "managed"
    unmanaged = "unmanaged"


class BootDiagnostics:
    """
    # Arguments
    is_enabled: bool
    storage_uri = str
    type: StorageType
    """

    def __init__(
            self,
            is_enabled: bool = none,
            storage_uri: str = none,
            type: StorageType = none):
        self.is_enabled = is_enabled
        self.storage_uri = storage_uri
        self.type = type


class DataDiskType(Enum):
    standard_lrs = "Standard_LRS"
    premium_lrs = "Premium_LRS"
    standard_ssd_lrs = "StandardSSD_LRS"
    ultra_ssd_lrs = "UltraSSD_LRS"


class CachingType(Enum):
    none = "None"
    read_only = "ReadOnly"
    read_write = "ReadWrite"


class DataDisk:
    """
    # Arguments
    lun: int
    size_g_b = int
    type: DataDiskType
    caching : CachingType
    """

    def __init__(
            self,
            lun: int = none,
            size_g_b: int = none,
            type: DataDiskType = none,
            caching: CachingType = none):
        self.lun = lun
        self.size_g_b = size_g_b
        self.type = type
        self.caching = caching


class Extension:
    """
    # Arguments
    api_version: str
    minor_version_auto_upgrade: bool
    name: str
    protected_settings: ProtectedSettings
    public_settings: PublicSettings
    publisher: str
    type: str
    """

    def __init__(
            self,
            api_version: str = none,
            minor_version_auto_upgrade: bool = none,
            name: str = none,
            protected_settings=none,
            public_settings=none,
            publisher: str = none,
            type: str = none):
        self.api_version = api_version
        self.minor_version_auto_upgrade = minor_version_auto_upgrade
        self.name = name
        self.protected_settings = protected_settings
        self.public_settings = public_settings
        self.publisher = publisher
        self.type = type


class Marketplace:
    """
    # Arguments
    publisher: str
    offer: str
    sku: str
    version: str
    """

    def __init__(
            self,
            publisher: str = none,
            offer: str = none,
            sku: str = none,
            version: str = none):
        self.publisher = publisher
        self.offer = offer
        self.sku = sku
        self.version = version


class Custom:
    """
    # Arguments
    resource_group_name: str
    name: str
    """

    def __init__(
            self,
            resource_group_name: str = none,
            name: str = none):
        self.resource_group_name = resource_group_name
        self.name = name


class Gallery:
    """
    # Arguments
    gallery_name: str
    image_name: str
    resource_group_name: str
    spot_account_id: str
    version_name: str
    """

    def __init__(
            self,
            gallery_name: str = none,
            image_name: str = none,
            resource_group_name: str = none,
            spot_account_id: str = none,
            version_name: str = none):
        self.gallery_name = gallery_name
        self.image_name = image_name
        self.resource_group_name = resource_group_name
        self.spot_account_id = spot_account_id
        self.version_name = version_name


class Image:
    """
    # Arguments
    marketplace: Marketplace
    custom: Custom
    gallery: Gallery
    """

    def __init__(self, marketplace: Marketplace = none, custom: Custom = none, gallery: Gallery = none):
        self.marketplace = marketplace
        self.custom = custom
        self.gallery = gallery


class LoadBalancerType(Enum):
    load_balancer = "loadBalancer"
    application_gateway = "applicationGateway"


class LoadBalancer:
    """
    # Arguments
    backend_pool_names: List[str]
    load_balancer_sku: str
    name: str
    resource_group_name: str
    type: LoadBalancerType
    """

    def __init__(
            self,
            backend_pool_names: List[str] = none,
            load_balancer_sku: str = none,
            name: str = none,
            resource_group_name: str = none,
            type: LoadBalancerType = none):
        self.backend_pool_names = backend_pool_names
        self.load_balancer_sku = load_balancer_sku
        self.name = name
        self.resource_group_name = resource_group_name
        self.type = type


class LoadBalancerConfig:
    """
    # Arguments
    load_balancers: List[LoadBalancer]
    """

    def __init__(self, load_balancers: List[LoadBalancer] = none):
        self.load_balancers = load_balancers


class Login:
    """
    # Arguments
    ssh_public_key: str
    user_name: str
    password: str
    """

    def __init__(
            self,
            ssh_public_key: str = none,
            user_name: str = none,
            password: str = none):
        self.ssh_public_key = ssh_public_key
        self.user_name = user_name
        self.password = password


class ManagedServiceIdentity:
    """
    # Arguments
    resource_group_name: str
    name: str
    """

    def __init__(
            self,
            resource_group_name: str = none,
            name: str = none):
        self.resource_group_name = resource_group_name
        self.name = name


class PrivateIpAddressVersion(Enum):
    ipv4 = "IPv4"
    ipv6 = "IPv6"


class AdditionalIpConfiguration:
    """
    # Arguments
    name: str
    private_ip_address_version: PrivateIpAddressVersion
    """

    def __init__(
            self,
            name: str = none,
            private_ip_address_version: PrivateIpAddressVersion = none):
        self.name = name
        self.private_ip_address_version = private_ip_address_version


class ApplicationSecurityGroup:
    """
    # Arguments
    name: str
    resource_group_name: str
    """

    def __init__(
            self,
            name: str = none,
            resource_group_name: str = none):
        self.name = name
        self.resource_group_name = resource_group_name


class NetworkSecurityGroup:
    """
    # Arguments
    name: str
    resource_group_name: str
    """

    def __init__(
            self,
            name: str = none,
            resource_group_name: str = none):
        self.name = name
        self.resource_group_name = resource_group_name


class PublicIp:
    """
    # Arguments
    name: str
    resource_group_name: str
    """

    def __init__(
            self,
            name: str = none,
            resource_group_name: str = none):
        self.name = name
        self.resource_group_name = resource_group_name


class PublicIpSku(Enum):
    standard = "Standard"
    basic = "Basic"


class NetworkInterface:
    """
    # Arguments
    additional_ip_configurations: List[AdditionalIpConfiguration]
    application_security_groups: List[ApplicationSecurityGroup]
    assign_public_ip: bool
    enable_ip_forwarding: bool
    is_primary: bool
    network_security_group: NetworkSecurityGroup
    private_ip_addresses: List[str]
    public_ips: List[PublicIp]
    public_ip_sku: str
    subnet_name: str
    """

    def __init__(
            self,
            additional_ip_configurations: List[AdditionalIpConfiguration] = none,
            application_security_groups: List[ApplicationSecurityGroup] = none,
            assign_public_ip: bool = none,
            enable_ip_forwarding: bool = none,
            is_primary: bool = none,
            network_security_group: NetworkSecurityGroup = none,
            private_ip_addresses: List[str] = none,
            public_ips: List[PublicIp] = none,
            public_ip_sku: PublicIpSku = none,
            subnet_name: str = none):
        self.additional_ip_configurations = additional_ip_configurations
        self.application_security_groups = application_security_groups
        self.assign_public_ip = assign_public_ip
        self.enable_ip_forwarding = enable_ip_forwarding
        self.is_primary = is_primary
        self.network_security_group = network_security_group
        self.private_ip_addresses = private_ip_addresses
        self.public_ips = public_ips
        self.public_ip_sku = public_ip_sku
        self.subnet_name = subnet_name


class Network:
    """
    # Arguments
    network_interfaces: List[NetworkInterface]
    resource_group_name: str
    virtual_network_name: str
    """

    def __init__(
            self,
            network_interfaces: List[NetworkInterface] = none,
            virtual_network_name: str = none,
            resource_group_name: str = none):
        self.network_interfaces = network_interfaces
        self.virtual_network_name = virtual_network_name
        self.resource_group_name = resource_group_name


class OsDisk:
    """
    # Arguments
    size_g_b: int
    type: DataDiskType
    caching : CachingType
    """

    def __init__(
            self,
            size_g_b: int = none,
            type: DataDiskType = none,
            caching: CachingType = none):
        self.size_g_b = size_g_b
        self.type = type
        self.caching = caching


class SourceVault:
    """
    # Arguments
    name: str
    resource_group_name: str
    """

    def __init__(
            self,
            name: str = none,
            resource_group_name: str = none):
        self.name = name
        self.resource_group_name = resource_group_name


class VaultCertificate:
    """
    # Arguments
    certificate_store: str
    certificate_url: str
    """

    def __init__(
            self,
            certificate_store: str = none,
            certificate_url: str = none):
        self.certificate_store = certificate_store
        self.certificate_url = certificate_url


class Secret:
    """
    # Arguments
    source_vault: SourceVault
    vault_certificates: List[VaultCertificate]
    """

    def __init__(
            self,
            source_vault: SourceVault = none,
            vault_certificates: List[VaultCertificate] = none):
        self.source_vault = source_vault
        self.vault_certificates = vault_certificates


class Tag:
    """
    # Arguments
    tag_key: str
    tag_value: str
    """

    def __init__(
            self,
            tag_key: str = none,
            tag_value: str = none):
        self.tag_key = tag_key
        self.tag_value = tag_value


class VmSizes:
    """
    # Arguments
    od_sizes: List[str]
    preferred_spot_sizes: List[str]
    spot_sizes: List[str]
    """

    def __init__(
            self,
            od_sizes: List[str] = none,
            preferred_spot_sizes: List[str] = none,
            spot_sizes: List[str] = none):
        self.od_sizes = od_sizes
        self.preferred_spot_sizes = preferred_spot_sizes
        self.spot_sizes = spot_sizes


class ProximityPlacementGroups:
    """
    # Arguments
    name: str
    resource_group_name: str
    """

    def __init__(
            self,
            name: str = none,
            resource_group_name: str = none):
        self.name = name
        self.resource_group_name = resource_group_name


class SecurityType(Enum):
    standard = "Standard"
    trusted_launch = "TrustedLaunch"


class Security:
    """
    # Arguments
    secure_boot_enabled: bool
    security_type: SecurityType
    v_tpm_enabled: bool
    """

    def __init__(
            self,
            secure_boot_enabled: bool = none,
            security_type: SecurityType = none,
            v_tpm_enabled: bool = none):
        self.secure_boot_enabled = secure_boot_enabled
        self.security_type = security_type
        self.v_tpm_enabled = v_tpm_enabled


class LaunchSpecification:
    """
    # Arguments
    boot_diagnostics: BootDiagnostics
    custom_data: str
    data_disks: List[DataDisk]
    extensions: List[Extension]
    image: Image
    license_type: str
    load_balancers_config: LoadBalancerConfig
    login: Login
    managed_service_identities: List[ManagedServiceIdentity]
    network: Network
    os_disk: OsDisk
    proximity_placement_groups: List[ProximityPlacementGroups]
    secrets: List[Secret]
    security: Security
    shutdown_script: str
    tags: List[Tag]
    user_data: str
    vm_name: str
    vm_name_prefix: str
    """

    def __init__(
            self,
            boot_diagnostics: BootDiagnostics = none,
            custom_data: str = none,
            data_disks: List[DataDisk] = none,
            extensions: List[Extension] = none,
            image: Image = none,
            license_type: str = none,
            load_balancers_config: LoadBalancerConfig = none,
            login: Login = none,
            managed_service_identities: List[ManagedServiceIdentity] = none,
            network: Network = none,
            os_disk: OsDisk = none,
            proximity_placement_groups: List[ProximityPlacementGroups] = none,
            secrets: List[Secret] = none,
            security: Security = none,
            shutdown_script: str = none,
            tags: List[Tag] = none,
            user_data: str = none,
            vm_name: str = none,
            vm_name_prefix: str = none):
        self.boot_diagnostics = boot_diagnostics
        self.custom_data = custom_data
        self.data_disks = data_disks
        self.extensions = extensions
        self.image = image
        self.license_type = license_type
        self.load_balancers_config = load_balancers_config
        self.login = login
        self.managed_service_identities = managed_service_identities
        self.network = network
        self.os_disk = os_disk
        self.proximity_placement_groups = proximity_placement_groups
        self.secrets = secrets
        self.security = security
        self.shutdown_script = shutdown_script
        self.tags = tags
        self.user_data = user_data
        self.vm_name = vm_name
        self.vm_name_prefix = vm_name_prefix


class OsType(Enum):
    linux = "Linux"
    windows = "Windows"


class Compute:
    """
    # Arguments
    launch_specification: LaunchSpecification
    os: OsType
    preferred_zone: str
    vm_sizes: VmSizes
    zones: List[str]
    """

    def __init__(
            self,
            launch_specification: LaunchSpecification = none,
            os: OsType = none,
            preferred_zone: str = none,
            vm_sizes: VmSizes = none,
            zones: List[str] = none):
        self.launch_specification = launch_specification
        self.os = os
        self.preferred_zone = preferred_zone
        self.vm_sizes = vm_sizes
        self.zones = zones


# endregion

# region StatefulNode


class StatefulNode:
    """
    # Arguments
    compute: Compute
    description: str
    health: Health
    name: str
    persistence: Persistence
    region: str
    resource_group_name: str
    scheduling: Scheduling
    strategy: Strategy
    """

    def __init__(
            self,
            compute: Compute = none,
            description: str = none,
            health: Health = none,
            name: str = none,
            persistence: Persistence = none,
            region: str = none,
            resource_group_name: str = none,
            scheduling: Scheduling = none,
            strategy: Strategy = none):
        self.compute = compute
        self.description = description
        self.health = health
        self.name = name
        self.persistence = persistence
        self.region = region
        self.resource_group_name = resource_group_name
        self.scheduling = scheduling
        self.strategy = strategy


# endregion


# region Client Requests
class CreateStatefulNodeRequest:
    def __init__(self, stateful_node: StatefulNode):
        self.stateful_node = stateful_node

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class UpdateStatefulNodeRequest:
    def __init__(self, stateful_node: StatefulNode):
        self.stateful_node = stateful_node

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class Deallocate:
    def __init__(
            self,
            should_deallocate: bool = none,
            ttl_in_hours: int = none):
        self.should_deallocate = should_deallocate
        self.ttl_in_hours = ttl_in_hours

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class DeallocationConfig:
    def __init__(self,
                 disk_deallocation_config: Deallocate = none,
                 network_deallocation_config: Deallocate = none,
                 public_ip_deallocation_config: Deallocate = none,
                 snapshot_deallocation_config: Deallocate = none,
                 should_terminate_vm: bool = none):
        self.disk_deallocation_config = disk_deallocation_config
        self.network_deallocation_config = network_deallocation_config
        self.public_ip_deallocation_config = public_ip_deallocation_config
        self.snapshot_deallocation_config = snapshot_deallocation_config
        self.should_terminate_vm = should_terminate_vm

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class DeleteStatefulNodeRequest:
    def __init__(self, deallocation_config: DeallocationConfig = none):
        self.deallocation_config = deallocation_config

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ImportVmConfiguration:
    def __init__(self,
                 convert_unmanaged_disks: bool = none,
                 draining_timeout: int = none,
                 node: StatefulNode = none,
                 original_vm_name: str = none,
                 resource_group_name: str = none,
                 resource_retention_time: int = none):
        self.convert_unmanaged_disks = convert_unmanaged_disks
        self.draining_timeout = draining_timeout
        self.node = node
        self.original_vm_name = original_vm_name
        self.resource_group_name = resource_group_name
        self.resource_retention_time = resource_retention_time

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ImportVmToStatefulNodeRequest:
    def __init__(self, import_vm_configuration: ImportVmConfiguration = none):
        self.stateful_node_import = import_vm_configuration

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class AttachDataDiskConfiguration:
    def __init__(
            self,
            data_disk_name: str = none,
            data_disk_resource_group_name: str = none,
            lun: int = none,
            size_g_b: int = none,
            storage_account_type: DataDiskType = none,
            zone: str = none):
        self.data_disk_name = data_disk_name
        self.data_disk_resource_group_name = data_disk_resource_group_name
        self.lun = lun
        self.size_g_b = size_g_b
        self.storage_account_type = storage_account_type
        self.zone = zone


class AttachDataDiskToStatefulNodeRequest:
    def __init__(self, attach_data_disk_configuration: AttachDataDiskConfiguration = none):
        self.data_disk_name = attach_data_disk_configuration.data_disk_name
        self.data_disk_resource_group_name = attach_data_disk_configuration.data_disk_resource_group_name
        self.lun = attach_data_disk_configuration.lun
        self.size_g_b = attach_data_disk_configuration.size_g_b
        self.storage_account_type = attach_data_disk_configuration.storage_account_type
        self.zone = attach_data_disk_configuration.zone

    def toJSON(self):
        return json.dumps(
            self,
            default=lambda o: o.__dict__,
            sort_keys=True,
            indent=4)


class DetachDataDiskConfiguration:
    def __init__(
            self,
            data_disk_name: str = none,
            data_disk_resource_group_name: str = none,
            should_deallocate: bool = none,
            ttl_in_hours: int = none):
        self.data_disk_name = data_disk_name
        self.data_disk_resource_group_name = data_disk_resource_group_name
        self.should_deallocate = should_deallocate
        self.ttl_in_hours = ttl_in_hours


class DetachDataDiskFromStatefulNodeRequest:
    def __init__(self, detach_data_disk_configuration: DetachDataDiskConfiguration = none):
        self.data_disk_name = detach_data_disk_configuration.data_disk_name
        self.data_disk_resource_group_name = detach_data_disk_configuration.data_disk_resource_group_name
        self.should_deallocate = detach_data_disk_configuration.should_deallocate
        self.ttl_in_hours = detach_data_disk_configuration.ttl_in_hours

    def toJSON(self):
        return json.dumps(
            self,
            default=lambda o: o.__dict__,
            sort_keys=True,
            indent=4)


class SwapOsDiskConfiguration:
    def __init__(self, os_disk_name: str = none,
                 os_disk_resource_group_name: str = none,
                 retention_time: int = none,
                 should_terminate: bool = none):
        self.os_disk_name = os_disk_name
        self.os_disk_resource_group_name = os_disk_resource_group_name
        self.retention_time = retention_time
        self.should_terminate = should_terminate


class SwapOsDiskToStatefulNodeRequest:
    def __init__(self, config: SwapOsDiskConfiguration = none):
        self.os_disk_name = config.os_disk_name
        self.os_disk_resource_group_name = config.os_disk_resource_group_name
        self.retention_time = config.retention_time
        self.should_terminate = config.should_terminate

    def toJSON(self):
        return json.dumps(
            self,
            default=lambda o: o.__dict__,
            sort_keys=True,
            indent=4)


# endregion



import json
from enum import Enum

none = "d3043820717d74d9a17694c176d39733"


# region Rollouts
class SpotDeployment:
    """
    # Arguments
    name: str
    old_version: str
    new_version: str
    old_manifest: str
    new_manifest: str
    """

    def __init__(
            self,
            name: str = none,
            old_version: str = none,
            new_version: str = none,
            old_manifest: str = none,
            new_manifest: str = none):
        self.name = name
        self.old_version = old_version
        self.new_version = new_version
        self.old_manifest = old_manifest
        self.new_manifest = new_manifest


class Status(Enum):
    manual_pausing = 'manualPausing'
    deallocating = 'deallocating'
    manual_paused = 'manualPaused'
    canceled = 'canceled'
    pending = 'pending'
    in_progress = 'inProgress'
    finished = 'finished'
    failed = 'failed'
    paused = 'paused'
    aborting = 'aborting'
    aborted = 'aborted'


class Rollouts:
    """
    # Arguments
    spot_deployment: SpotDeployment
    namespace: str
    cluster_id: str
    original_rollout_id: str
    new_rollout_id: str
    strategy: str
    strategy_type: str
    strategy_name: str
    status: Status
    start_time: str
    end_time: str
    rollout_spec: str
    cloud_provider: str
    identity: str
    """

    def __init__(
            self,
            spot_deployment: SpotDeployment = none,
            namespace: str = none,
            cluster_id: str = none,
            original_rollout_id: str = none,
            new_rollout_id: str = none,
            strategy: str = none,
            strategy_type: str = none,
            strategy_name: str = none,
            status: Status = none,
            start_time: str = none,
            end_time: str = none,
            rollout_spec: str = none,
            cloud_provider: str = none,
            identity: str = none):
        self.spot_deployment = spot_deployment
        self.namespace = namespace
        self.cluster_id = cluster_id
        self.original_rollout_id = original_rollout_id
        self.new_rollout_id = new_rollout_id
        self.strategy = strategy
        self.strategy_type = strategy_type
        self.strategy_name = strategy_name
        self.status = status
        self.start_time = start_time
        self.end_time = end_time
        self.rollout_spec = rollout_spec
        self.cloud_provider = cloud_provider
        self.identity = identity
# endregion


# region Action
class Action(Enum):
    promote = 'promote'
    promote_full = 'promoteFull'
    pause = 'pause'
    abort = 'abort'
    retry = 'retry'
# endregion


# region Client Requests
class CreateRolloutActionRequest:
    """
    # Arguments
    action : Action
    """

    def __init__(self, action: Action):
        self.action = action

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion



import json

from spotinst_sdk2.client import Client
import spotinst_sdk2.models.mrscaler.aws as mrScaler_aws


class MrScalerAwsClient(Client):
    __base_emr_url = "/aws/emr/mrScaler"

    # region EMR
    def create_emr(self, emr):
        """
        Create an EMR 

        # Arguments
        emr (EMR): EMR Object

        # Returns
        (Object): Elastigroup API response 
        """
        emr = mrScaler_aws.EMRCreationRequest(emr)

        excluded_group_dict = self.exclude_missing(json.loads(emr.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        group_response = self.send_post(
            body=body_json,
            url=self.__base_emr_url,
            entity_name='emr')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def update_emr(self, emr_id, emr):
        """
        Update an existing EMR 

        # Arguments
        emr_id (String): EMR id
        emr (EMR): EMR Object

        # Returns
        (Object): Elastigroup API response 
        """
        emr = mrScaler_aws.EMRCreationRequest(emr)

        excluded_group_dict = self.exclude_missing(json.loads(emr.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        group_response = self.send_put(
            body=body_json,
            url=self.__base_emr_url +
            "/" + emr_id,
            entity_name='emr')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_emr(self):
        """
        Get all EMR in account

        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_get(
            url=self.__base_emr_url,
            entity_name="emr"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_emr(self, emr_id):
        """
        Get an existing EMR json

        # Arguments
        emr_id (String): EMR id

        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_get(
            url=self.__base_emr_url +
            "/" + emr_id,
            entity_name="emr"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_emr_instances(self, emr_id):
        """
        Get instances from EMR 

        # Arguments
        emr_id (String): EMR id

        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_get(
            url=self.__base_emr_url +
            "/" + emr_id +
            "/instance",
            entity_name="emr"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_emr_cluster(self, emr_id):
        """
        Get cluster from EMR

        # Arguments
        emr_id (String): EMR id

        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_get(
            url=self.__base_emr_url +
            "/" + emr_id +
            "/cluster",
            entity_name="emr"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_emr_cost(self, emr_id, from_date=None, to_date=None):
        """
        Get cost from EMR

        # Arguments
        emr_id (String): EMR id
        from_date (String) (Optional): From Date
        to_date (String) (Optional): to date

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict(fromDate=from_date, toDate=to_date)

        response = self.send_get(
            url=self.__base_emr_url +
            "/" + emr_id +
            "/costs",
            query_params=query_params,
            entity_name="emr"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def delete_emr(self, emr_id):
        """
        Delete an EMR

        # Arguments
        emr_id (String): EMR id

        # Returns
        (Object): Elastigroup API response 
        """
        response = self.send_delete(
            url=self.__base_emr_url +
            "/" + emr_id,
            entity_name="emr"
        )

        return response

    def scale_up_emr(self, emr_id, adjustment):
        """
        Scale up an EMR

        # Arguments
        emr_id (String): EMR id
        adjustment (Int): Ammount to scale

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict(adjustment=adjustment)

        response = self.send_put(
            url=self.__base_emr_url +
            "/" + emr_id +
            "/scale/up",
            query_params=query_params,
            entity_name="emr"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def scale_down_emr(self, emr_id, adjustment):
        """
        Scale down an EMR

        # Arguments
        emr_id (String): EMR id
        adjustment (Int): Ammount to scale

        # Returns
        (Object): Elastigroup API response 
        """
        query_params = dict(adjustment=adjustment)

        response = self.send_put(
            url=self.__base_emr_url +
            "/" + emr_id +
            "/scale/down",
            query_params=query_params,
            entity_name="emr"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    # endregion



import json
from typing import List

from spotinst_sdk2.client import Client
import spotinst_sdk2.models.ocean.aws as aws_ocean
import spotinst_sdk2.models.ocean.azure as azure_ocean

# region AWS


class OceanAwsClient(Client):
    __base_ocean_cluster_url = "/ocean/aws/k8s/cluster"
    __base_ocean_launchspec_url = "/ocean/aws/k8s/launchSpec"
    __base_ocean_url = "/ocean/k8s/cluster/"
    __base_ocean_extended_resource_definition_url = "/ocean/k8s/extendedResourceDefinition"

    def create_ocean_cluster(self, ocean: aws_ocean.Ocean):
        """
        Create an Ocean Cluster 

        # Arguments
        ocean (Ocean): Ocean Object

        # Returns
        (Object): Ocean API response
        """
        ocean = aws_ocean.OceanRequest(ocean)

        excluded_missing_dict = self.exclude_missing(
            json.loads(ocean.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url,
            entity_name='ocean')

        formatted_response = self.convert_json(response,
                                               self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def update_ocean_cluster(self, ocean_id: str, ocean: aws_ocean.Ocean, auto_apply_tags: str = None):
        """
        Update an existing Ocean Cluster 

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        auto_apply_tags (String): Option to update instance tags on the fly without rolling the cluster.
        ocean (Ocean): Ocean object

        # Returns
        (Object): Ocean API response
        """
        ocean = aws_ocean.OceanRequest(ocean)

        excluded_missing_dict = self.exclude_missing(
            json.loads(ocean.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_put_with_params(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id,
            entity_name='ocean',
            user_query_params=dict(autoApplyTags=auto_apply_tags))

        formatted_response = self.convert_json(
            response,
            self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_ocean_cluster(self):
        """
        List the configurations for all Ocean clusters in the specified account.

        # Returns
        (Object): Ocean API response
        """

        response = self.send_get(
            url=self.__base_ocean_cluster_url,
            entity_name="ocean"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_all_ocean_sizing(self, ocean_id: str, namespace: str):
        """
        Get all right sizing recommendations for an Ocean cluster

        # Returns
        (Object): Ocean API response
        """
        response = self.send_get(
            url=(self.__base_ocean_cluster_url +
                 "/" + ocean_id +
                 "/rightSizing/suggestion?namespace={}"
                 ).format(namespace),
            entity_name="ocean"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def fetch_rightsizing_recommendations(self, ocean_id: str, filter: aws_ocean.RightSizingRecommendationFilter = None):
        """
        Get right-sizing recommendations for an Ocean cluster and filter them according to namespace or label.

        # Arguments
        ocean_id (String): Id of the Ocean Cluster
        filter (RightSizingRecommendationFilter): Optional - may be null.

        # Returns
        (Object): Ocean API response
        """
        recommendation_request = aws_ocean.RightSizingRecommendationRequest(
            filter)

        excluded_missing_dict = self.exclude_missing(
            json.loads(recommendation_request.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        group_response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url +
            "/" + ocean_id + "/rightSizing/suggestion",
            entity_name='ocean')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_ocean_cluster(self, ocean_id: str):
        """
        Get an existing Ocean Cluster json

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): Ocean API response
        """
        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id,
            entity_name="ocean"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def delete_ocean_cluster(self, ocean_id: str):
        """
        Delete an Ocean Cluster

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): Ocean API response
        """
        return self.send_delete(
            url=self.__base_ocean_cluster_url + "/" + ocean_id,
            entity_name="ocean"
        )

    def get_aggregated_cluster_costs(self, ocean_id: str, aggregated_cluster_costs: aws_ocean.AggregatedClusterCosts):
        """
        Get aggregated cluster costs

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        aggregated_cluster_costs (AggregatedClusterCosts): Aggregated Cluster Costs request

        # Returns
        (Object): Aggregated Cluster Costs API response
        """
        aggregated_cluster_costs_request = aws_ocean.AggregatedClusterCostRequest(
            aggregated_cluster_costs)

        excluded_missing_dict = self.exclude_missing(
            json.loads(aggregated_cluster_costs_request.toJSON()))

        formatted_missing_dict = self.convert_json_with_list_of_lists(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        aggregated_costs_response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/aggregatedCosts",
            entity_name='ocean (aggregated cluster costs)')

        formatted_response = self.convert_json(
            aggregated_costs_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def initiate_roll(self, ocean_id: str, cluster_roll: aws_ocean.Roll):
        """
        Initiate Cluster Rolls

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        cluster_roll (Roll): Cluster Roll / Roll with Instance Ids/ Launch specification Ids

        # Returns
        (Object): Cluster Roll API response
        """
        roll_request = aws_ocean.ClusterRollInitiateRequest(cluster_roll)

        excluded_missing_dict = self.exclude_missing(
            json.loads(roll_request.toJSON()))

        formatted_missing_dict = self.convert_json_with_list_of_lists(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        rolls_response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/roll",
            entity_name='ocean (Cluster Roll)')

        formatted_response = self.convert_json(
            rolls_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def list_rolls(self, ocean_id: str):
        """
        Get status for all rolls of an Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): List of Cluster Roll API response
        """
        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/roll",
            entity_name="ocean (Cluster Roll)"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def update_roll(self, ocean_id: str, roll_id: str, status: str):
        """
        Update a roll of an Ocean cluster.
        Performing the request will stop the next batch in a roll.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        roll_id (String): Ocean cluster roll identifier
        update_roll (UpdateRoll): update roll request

        # Returns
        (Object): Cluster Roll API response
        """
        update_roll_request = aws_ocean.ClusterRollUpdateRequest(status)

        excluded_missing_dict = self.exclude_missing(
            json.loads(update_roll_request.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/roll/" + roll_id,
            entity_name='ocean (Cluster Roll)')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_roll(self, ocean_id: str, roll_id: str):
        """
        Get status for a roll of an Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        account_id (String): The ID of the account associated with your token.
        roll_id (String): Ocean cluster roll identifier

        # Returns
        (Object): Cluster Roll API response
        """
        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/roll/" + roll_id,
            entity_name="ocean (Cluster Roll)"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_cluster_nodes(self, ocean_id: str, instance_id: str = None, launch_spec_id: str = None):
        """
        Get nodes data of an Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        instance_id (String): Get a specific node by instance id
        launch_spec_id (String): Ocean cluster launch specification identifier.

        # Returns
        (Object): Ocean Kubernetes AWS Nodes Data response
        """
        query_params = dict(instanceId=instance_id,
                            launchSpecId=launch_spec_id)

        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/nodes",
            entity_name="ocean (Cluster Nodes)",
            query_params=query_params
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_heartbeat_status(self, ocean_id: str):
        """
        Get the heartbeat status of the Ocean Controller for the cluster.
        The response returns the heartbeat status and the last heartbeat timestamp.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): Ocean Get Heartbeat response
        """
        response = self.send_get(
            url=self.__base_ocean_url + ocean_id + "/controllerHeartbeat",
            entity_name="ocean (Cluster Heartbeat)"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def instance_types_filter_simulation(self, ocean_id: str,
                                         instance_type_filter: aws_ocean.InstanceTypesFilters):
        """
        Returns all instances types that match the given filters.
        These instance types will be used if the cluster is configured with these filters.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        instance_type_filter (InstanceTypesFilters): The Instance types that match with all filters compose the
        Ocean's whitelist parameter. Cannot be configured together with whitelist/blacklist.

        # Returns
        (Object): Ocean Instance Types Filter Simulation response
        """
        group_dict = aws_ocean.InstanceTypesFilterSimulationRequest(
            instance_type_filter)

        excluded_missing_dict = self.exclude_missing(
            json.loads(group_dict.toJSON()))

        formatted_missing_dict = self.convert_json_with_list_of_lists(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        aggregated_costs_response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" +
            ocean_id + "/instanceTypeFiltersSimulation",
            entity_name='ocean')

        formatted_response = self.convert_json(
            aggregated_costs_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def allowed_instance_types(self, ocean_id: str):
        """
        Return the list of the allowed Ocean cluster instance types.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): Ocean Allowed Instance Types response
        """
        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/allowedInstanceTypes",
            entity_name="ocean"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_virtual_node_group(self, ocean_launch_spec_id: str):
        """
        Get Virtual Node Group of the cluster

        # Arguments
        ocean_launch_spec_id (String): Ocean cluster launch specification identifier

        # Returns
        (Object): Ocean Allowed Instance Types response
        """
        response = self.send_get(
            url=self.__base_ocean_launchspec_url + "/" + ocean_launch_spec_id,
            entity_name="ocean"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def launch_nodes_in_vng(self, ocean_launch_spec_id: str, amount: int):
        """
        Launch nodes in Virtual Node Group.

        # Arguments
        ocean_launch_spec_id (String): Ocean cluster launch specification identifier.
        amount (int): The number of nodes to launch.

        # Returns
        (Object): Ocean Virtual Node Group Launch API response
        """
        launch_node_request = aws_ocean.LaunchNodesRequest(amount)

        excluded_missing_dict = self.exclude_missing(
            json.loads(launch_node_request.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_ocean_launchspec_url + "/" +
            ocean_launch_spec_id + "/launchNodes",
            entity_name='ocean (Cluster Roll)')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def fetch_elastilog(self, ocean_id, from_date, to_date, severity=None, resource_id=None, limit=None):
        """
        Create an Ocean configuration according to an AWS autoscaling group (ASG) configuration.

        # Arguments
        to_date (String): to date
        from_date (String): to date
        severity(String) (Optional): Log level severity
        resource_id(String) (Optional): Filter log extracted entires related to a
          specific resource id
        limit(String) (Optional): Maximum number of lines to extract in a response

        # Returns
        (Object): Ocean Get Log API response
        """
        geturl = self.__base_ocean_cluster_url + "/" + ocean_id + "/log"
        query_params = dict(toDate=to_date, fromDate=from_date, severity=severity,
                            resourceId=resource_id, limit=limit)

        result = self.send_get(
            url=geturl, entity_name='ocean_aws_log', query_params=query_params)

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def import_asg_to_ocean_cluster(self, auto_scaling_group_name, region, import_asg_to_ocean_cluster: aws_ocean.ImportASGToOceanCluster):
        """
        Create an Ocean configuration according to an AWS autoscaling group (ASG) configuration.

        # Arguments
        auto_scaling_group_name (String): The ASG name
        region (String): region
        import_asg_to_ocean_cluster (ImportASGToOceanCluster): ImportASGToOceanCluster Object

        # Returns
        (Object): Ocean Import ASG to Ocean Response
        """
        import_asg_to_ocean_cluster_body = aws_ocean.ImportASGToOceanClusterRequest(
            import_asg_to_ocean_cluster)

        excluded_missing_dict = self.exclude_missing(
            json.loads(import_asg_to_ocean_cluster_body.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        geturl = self.__base_ocean_cluster_url + "/autoScalingGroup/import"

        query_params = dict(
            autoScalingGroupName=auto_scaling_group_name, region=region)

        result = self.send_post_with_params(
            body=body_json,
            url=geturl,
            entity_name='import_asg_to_ocean_cluster',
            user_query_params=query_params)

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def create_virtual_node_group(self, vng: aws_ocean.VirtualNodeGroup, initial_nodes: int = None):
        """
        Create a VNG inside ocean cluster

        # Arguments
        vng (VirtualNodeGroup): VirtualNodeGroup Object
        initial_nodes: When set to an integer greater than 0, a corresponding number of nodes will be launched from the virtual node group created.

        # Returns
        (Object): Ocean Launch Spec response
        """
        ocean = aws_ocean.VNGRequest(vng)

        excluded_missing_dict = self.exclude_missing(
            json.loads(ocean.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        query_params = dict(initialNodes=initial_nodes)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_post_with_params(
            body=body_json,
            url=self.__base_ocean_launchspec_url,
            entity_name='ocean_aws_vng',
            user_query_params=query_params)

        formatted_response = self.convert_json(response,
                                               self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def update_virtual_node_group(self, vng_id: str, vng: aws_ocean.VirtualNodeGroup, auto_apply_tags: bool = None):
        """
        Update an existing VNG inside an Ocean Cluster 

        # Arguments
        vng_id (String): ID of the Ocean Virtual Node Group
        ocean (Ocean): Ocean object

        # Returns
        (Object): Ocean Launch Spec response
        """
        ocean = aws_ocean.VNGRequest(vng)

        excluded_missing_dict = self.exclude_missing(
            json.loads(ocean.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_put_with_params(
            body=body_json,
            url=self.__base_ocean_launchspec_url + "/" + vng_id,
            entity_name='ocean_aws_vng',
            user_query_params=dict(autoApplyTags=auto_apply_tags))

        formatted_response = self.convert_json(
            response,
            self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_virtual_node_groups(self, ocean_id: str = None):
        """
        List the configurations for all virtual node groups in the account
        or in a specified cluster.

        # Returns
        (Object): Ocean Launch Spec response
        """

        response = self.send_get(
            url=self.__base_ocean_launchspec_url,
            entity_name="ocean_aws_vng",
            query_params=dict(oceanId=ocean_id)
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def delete_virtual_node_group(self, vng_id: str, delete_nodes: bool = None, force_delete: bool = None):
        """
        Delete an Ocean Cluster

        # Arguments
        vng_id (String): ID of the Ocean VNG
        delete_nodes (Bool): When set to "true", all instances belonging to the deleted launch specification will be drained, detached, and terminated.
        force_delete (Bool): When set to "true", delete even if it is the only custom launch spec remaining, and default launch spec has useAsTemplateOnly = true.

        # Returns
        (Object): Ocean Launch Specification Delete response
        """
        return self.send_delete_with_params(
            url=self.__base_ocean_launchspec_url + "/" + vng_id,
            entity_name="ocean_aws_vng",
            user_query_params=dict(
                deleteNodes=delete_nodes, forceDelete=force_delete)
        )

    def attach_load_balancers(self, ocean_id: str, load_balancers):
        """
        Add new load balancers to the existing load balancers on the Ocean Cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        load_balancers (List[LoadBalancer]): Load balancers to add to the Ocean cluster.

        # Returns
        (Object): Ocean Attach Load Balancers API response
        """

        load_balancers = aws_ocean.LoadBalancersRequest(load_balancers)

        excluded_missing_dict = self.exclude_missing(
            json.loads(load_balancers.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        return self.send_put(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/loadBalancer/attach",
            entity_name="ocean_aws_attach_load_balancer",
            body=body_json
        )

    def detach_load_balancers(self, ocean_id, load_balancers):
        """
        Delete an Ocean Cluster

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        load_balancers (List[LoadBalancer]): Load balancers to remove from the Ocean cluster.

        # Returns
        (Object): Ocean Detach Load Balancers API response
        """

        load_balancers = aws_ocean.LoadBalancersRequest(load_balancers)

        excluded_missing_dict = self.exclude_missing(
            json.loads(load_balancers.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        return self.send_put(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/loadBalancer/detach",
            entity_name="ocean_aws_detach_load_balancer",
            body=body_json
        )

    def update_elastigroup_to_ocean(self, group_id: str):
        """
        Upgrade an Elastigroup with Kubernetes integration to Ocean for Kubernetes cluster. 

        # Arguments
        group_id (str): Elastigroup identifier

        # Returns
        (Object): Ocean API response
        """

        query_params = dict(groupId=group_id)

        response = self.send_post_with_params(
            body=None,
            url=self.__base_ocean_cluster_url+"/import",
            entity_name='ocean_aws_update_eg_to_ocean',
            user_query_params=query_params)

        formatted_response = self.convert_json(response,
                                               self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def detach_instances(self, ocean_id: str, detach_instances: aws_ocean.DetachInstances):
        """
        Detach instances from your Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        detach_instances (DetachInstances): Detach Instances Request

        # Returns
        (Object): Detach Instances API response
        """
        detach_instances_request = aws_ocean.DetachInstancesRequest(
            detach_instances)

        excluded_missing_dict = self.exclude_missing(
            json.loads(detach_instances_request.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        detach_instances_response = self.send_put(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/detachInstances",
            entity_name='ocean_aws_detach_instances)')

        formatted_response = self.convert_json(
            detach_instances_response, self.camel_to_underscore)

        return formatted_response["response"]

    def instance_types_filter_simulation_for_vng(self, launch_spec_id: str,
                                                 instance_types_filters: aws_ocean.InstanceTypesFilters):
        """
        Returns all instance types that match the given filters. These instance types will be used if the Virtual Node Group is configured with these filters.

        # Arguments
        launch_spec_id (String): Ocean cluster launch specification identifier.
        instance_type_filters (InstanceTypesFilters): List of instance types filters. The instance types that match with all filters compose the Virtual Node Group's instanceTypes parameter.
        The architectures that come from the Virtual Node Group's images will be taken into account when using this parameter.
        Cannot be configured together with Virtual Node Group's instanceTypes and with the Cluster's whitelist/blacklist/filters.

        # Returns
        (Object): Ocean Instance Types Filter Simulation response
        """
        group_dict = aws_ocean.InstanceTypesFilterSimulationRequestForVNG(
            instance_types_filters)

        excluded_missing_dict = self.exclude_missing(
            json.loads(group_dict.toJSON()))

        formatted_missing_dict = self.convert_json_with_list_of_lists(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        aggregated_costs_response = self.send_post(
            body=body_json,
            url=self.__base_ocean_launchspec_url + "/" +
            launch_spec_id + "/instanceTypeFiltersSimulation",
            entity_name='ocean_instance_type_filters_simulation_for_vng')

        formatted_response = self.convert_json(
            aggregated_costs_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def import_asg_to_ocean_vng(self, auto_scaling_group_name, ocean_id: str, import_asg_to_ocean_launch_spec: aws_ocean.ImportASGToOceanVNG):
        """
        Returns an Ocean Virtual Node Group (VNG) configuration in a given AWS autoscaling group (ASG). The returned value ("Imported VNG") can then be used as input to the Create Virtual Node Group API in order to create an actual VNG in your Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        auto_scaling_group_name (String): The ASG name.
        import_asg_to_ocean_launch_spec (ImportASGToOceanVNG): ImportASGToOceanVNG object 

        # Returns
        (Object): Ocean Import ASG to Launch Spec Response
        """

        import_asg_to_ocean_launch_spec_body = aws_ocean.ImportASGToOceanVNGRequest(
            import_asg_to_ocean_launch_spec)

        excluded_missing_dict = self.exclude_missing(
            json.loads(import_asg_to_ocean_launch_spec_body.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        query_params = dict(
            autoScalingGroupName=auto_scaling_group_name, oceanId=ocean_id)

        response = self.send_post_with_params(
            body=body_json,
            url=self.__base_ocean_launchspec_url + "/autoScalingGroup/import",
            entity_name="ocean_aws_asg_ocean_vng_import",
            user_query_params=query_params
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def allowed_instance_types_by_filters_for_vng(self, launch_spec_id: str):
        """
        Returns the Virtual Node Group's instance types when instance types filters is set.

        # Arguments
        launch_spec_id (String): Ocean Cluster Launch Specification Identifier

        # Returns
        (Object): Ocean Allowed Instance Types response
        """
        response = self.send_get(
            url=self.__base_ocean_launchspec_url + "/" +
            launch_spec_id + "/allowedInstanceTypesByFilters",
            entity_name="ocean_aws_allowed_instance_types_by_filters_for_vng"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def import_eks_cluster_node_group_to_ocean_vng(self, eks_cluster_name: str, eks_node_group_name: str, import_eks_node_group_to_ocean_launch_spec: aws_ocean.ImportEKSNodeGroupToOceanVNG,  ocean_id: str = None, region: str = None):
        """
        Returns an Ocean Virtual Node Group (VNG) configuration based on a given AWS EKS Cluster Node Group. 
        The returned value ("Imported VNG") can then be used as input to the Create Virtual Node Group API in order to create an actual VNG in your Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        eks_cluster_name (String): Cluster name of the EKS cluster.
        eks_node_group_name (String): Node group name to import.
        region (String): Node group name to import.
        import_eks_node_group_to_ocean_launch_spec (ImportEKSNodeGroupToOceanVNG): ImportEKSNodeGroupToOceanVNG object

        # Returns
        (Object): Ocean Import EKS Cluster Node Group to Launch Spec Response
        """

        import_eks_node_group_to_ocean_launch_spec_body = aws_ocean.ImportEKSNodeGroupToOceanVNGRequest(
            import_eks_node_group_to_ocean_launch_spec)

        excluded_missing_dict = self.exclude_missing(json.loads(
            import_eks_node_group_to_ocean_launch_spec_body.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        query_params = dict(eksClusterName=eks_cluster_name,
                            eksNodeGroupName=eks_node_group_name, region=region, oceanId=ocean_id)

        response = self.send_post_with_params(
            body=body_json,
            url=self.__base_ocean_launchspec_url + "/eksNodeGroup/import",
            entity_name="ocean_aws_eks_ocean_vng_import",
            user_query_params=query_params
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def create_migration(self, ocean_id: str, migration: aws_ocean.Migration):
        """
        Create a migration for a given existing instances.

        # Arguments
        migration (Migration): Migration Object

        # Returns
        (Object): Migration create response
        """
        migration = aws_ocean.MigrationRequest(migration)

        excluded_missing_dict = self.exclude_missing(
            json.loads(migration.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url+"/"+ocean_id+"/migration",
            entity_name='ocean_aws_migration')

        formatted_response = self.convert_json(response,
                                               self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_migration_discovery(self, ocean_id: str, should_fetch_pods: bool):
        """
        Get information about nodes which can be migrated into Ocean.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        should_fetch_pods (bool): Should fetch data about running pods for each node.

        # Returns
        (Object): Ocean API response
        """
        query_params = dict(shouldFetchPods=should_fetch_pods)

        response = self.send_get(
            url=self.__base_ocean_cluster_url+"/"+ocean_id+"/migration/discovery",
            entity_name="ocean_aws_migration",
            query_params=query_params
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def stop_migration(self, ocean_id: str, migration_id: str, migration: aws_ocean.Migration):
        """
        Stop an ongoing Workload Migration.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        migration_id (String): The migration identifier of a specific migration
        migration (Migration): Migration Update Configuration

        # Returns
        (Object): Ocean Migration response 
        """

        migration = aws_ocean.MigrationRequest(migration)

        excluded_missing_dict = self.exclude_missing(
            json.loads(migration.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_ocean_cluster_url+"/"+ocean_id+"/migration/"+migration_id,
            entity_name="ocean_aws_migration",
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_migration_status(self, ocean_id: str, migration_id: str):
        """
        Get Migration full info and status for an Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        migration_id (String): The migration identifier of a specific migration.

        # Returns
        (Object): Ocean API response 
        """

        response = self.send_get(
            url=self.__base_ocean_cluster_url+"/"+ocean_id+"/migration/"+migration_id,
            entity_name="ocean_aws_migration"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_migrations_summary(self, ocean_id: str = None):
        """
        Get summary of migrations history for an Ocean cluster.

        # Returns
        (Object): Ocean Migrations response 
        """

        response = self.send_get(
            url=self.__base_ocean_cluster_url+"/"+ocean_id+"/migration",
            entity_name="ocean_aws_migration",
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def create_extended_resource_definition(self, extended_resource_definition: aws_ocean.ExtendedResourceDefinition):
        """
        Creates an Ocean extended resource definition entity

        # Arguments
        extended_resource_definition (ExtendedResourceDefinition): The Ocean extended resource definition

        # Returns
        (Object): Ocean Create Extended Resource Definition response 
        """
        extended_resource_definition = aws_ocean.ExtendedResourceDefinitionRequest(
            extended_resource_definition)

        excluded_missing_dict = self.exclude_missing(
            json.loads(extended_resource_definition.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_ocean_extended_resource_definition_url,
            entity_name='ocean_aws_extended_resource_defintion')

        formatted_response = self.convert_json(response,
                                               self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_extended_resource_definition(self, ocean_extended_resource_definition_id: str):
        """
        Get Migration full info and status for an Ocean cluster.

        # Arguments
        ocean_extended_resource_definition_id (String): Identifier of the Ocean extended resource definition.

        # Returns
        (Object): Ocean Extended Resource Definition response 
        """

        response = self.send_get(
            url=self.__base_ocean_extended_resource_definition_url +
            "/"+ocean_extended_resource_definition_id,
            entity_name='ocean_aws_extended_resource_defintion'
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_extended_resource_definitions(self):
        """
        extended_resource_definition

        # Returns
        (Object): Ocean Extended Resource Defintion response 
        """

        response = self.send_get(
            url=self.__base_ocean_extended_resource_definition_url,
            entity_name="ocean_extended_resource_definition",
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def update_extended_resource_definition(self, ocean_extended_resource_definition_id: str, extended_resource_definition: aws_ocean.ExtendedResourceDefinition):
        """
        Only the mapping parameter is updatable for extended resource definition

        # Arguments
        ocean_extended_resource_definition_id (String): Identifier of the Ocean extended resource definition.
        extended_resource_definition (ExtendedResourceDefinition): The Ocean extended resource definition

        # Returns
        (Object): Ocean Extended Resource Definition response
        """
        extended_resource_definition = aws_ocean.ExtendedResourceDefinitionRequest(
            extended_resource_definition)

        excluded_missing_dict = self.exclude_missing(
            json.loads(extended_resource_definition.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_ocean_extended_resource_definition_url +
            "/" + ocean_extended_resource_definition_id,
            entity_name='ocean_extended_resource_defintion')

        formatted_response = self.convert_json(
            response,
            self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def delete_extended_resource_definition(self, ocean_extended_resource_definition_id: str):
        """
        Delete a specified Ocean extended resource definition.

        # Arguments
        ocean_extended_resource_definition_id (String): Identifier of the Ocean extended resource definition.

        # Returns
        (Object): Ocean Delete Extended Resource Definition response
        """
        return self.send_delete(
            url=self.__base_ocean_extended_resource_definition_url +
            "/" + ocean_extended_resource_definition_id,
            entity_name='ocean_extended_resource_defintion')
    # endregion


# region Azure
class OceanAzureClient(Client):
    __base_ocean_cluster_url = "/ocean/azure/np/cluster"
    __base_ocean_vng_url = "/ocean/azure/np/virtualNodeGroup"

    def create_ocean_cluster(self, ocean: azure_ocean.Ocean):
        """
        Create an Ocean Cluster 

        # Arguments
        ocean (Ocean): Ocean Object

        # Returns
        (Object): Ocean API response
        """
        ocean = azure_ocean.OceanRequest(ocean)

        excluded_missing_dict = self.exclude_missing(
            json.loads(ocean.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url,
            entity_name='ocean_aks')

        formatted_response = self.convert_json(response,
                                               self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_ocean_clusters(self):
        """
        List the configurations for all Ocean clusters in the specified account.

        # Returns
        (Object): Ocean API response
        """

        response = self.send_get(
            url=self.__base_ocean_cluster_url,
            entity_name="ocean_aks"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_ocean_cluster(self, ocean_id: str):
        """
        Get an existing Ocean Cluster json

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): Ocean API response
        """
        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id,
            entity_name="ocean_aks"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def delete_ocean_cluster(self, ocean_id: str):
        """
        Delete an Ocean Cluster

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): Ocean API response
        """
        return self.send_delete(
            url=self.__base_ocean_cluster_url + "/" + ocean_id,
            entity_name="ocean_aks"
        )

    def update_ocean_cluster(self, ocean_id: str, ocean: azure_ocean.Ocean):
        """
        Update an existing Ocean Cluster 

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        ocean (Ocean): Ocean object

        # Returns
        (Object): Ocean API response
        """
        ocean = azure_ocean.OceanRequest(ocean)

        excluded_missing_dict = self.exclude_missing(
            json.loads(ocean.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id,
            entity_name='ocean_aks')

        formatted_response = self.convert_json(
            response,
            self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def import_cluster_configuration(self, aks_cluster_name: str, resource_group_name: str):
        """
        Import cluster configuration of an AKS cluster to use in create_ocean_cluster api call

        # Arguments
        aks_cluster_name (String): Name of the AKS cluster
        resource_group_name (String): Resource Group Name 

        # Returns
        (Object): Ocean API response
        """

        response = self.send_post_with_params(
            body=None,
            url=self.__base_ocean_cluster_url+"/aks/import",
            entity_name='ocean_aks',
            user_query_params=dict(aksClusterName=aks_cluster_name, resourceGroupName=resource_group_name))

        formatted_response = self.convert_json(response,
                                               self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def create_ocean_vng(self, vng: azure_ocean.VirtualNodeGroupTemplate):
        """
        Create a VNG inside ocean cluster

        # Arguments
        vng (VirtualNodeGroup): VirtualNodeGroup Object

        # Returns
        (Object): Ocean API response
        """
        ocean = azure_ocean.VNGRequest(vng)

        excluded_missing_dict = self.exclude_missing(
            json.loads(ocean.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_ocean_vng_url,
            entity_name='ocean_aks_vng')

        formatted_response = self.convert_json(response,
                                               self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def update_ocean_vng(self, vng_id: str, vng: azure_ocean.VirtualNodeGroupTemplate):
        """
        Update an existing VNG inside an Ocean Cluster 

        # Arguments
        vng_id (String): ID of the Ocean Virtual Node Group
        ocean (Ocean): Ocean object

        # Returns
        (Object): Ocean VNG API response
        """
        ocean = azure_ocean.VNGRequest(vng)

        excluded_missing_dict = self.exclude_missing(
            json.loads(ocean.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_ocean_vng_url + "/" + vng_id,
            entity_name='ocean_aks_vng')

        formatted_response = self.convert_json(
            response,
            self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_ocean_vng(self, vng_id: str):
        """
        Get an existing Ocean Virtual Node Group json

        # Arguments
        vng_id (String): ID of the Ocean VNG

        # Returns
        (Object): Ocean VNG API response
        """
        response = self.send_get(
            url=self.__base_ocean_vng_url + "/" + vng_id,
            entity_name="ocean_aks_vng"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_ocean_vngs(self, ocean_id: str = None):
        """
        List the configurations for all virtual node groups in the account
        or in a specified cluster.

        # Returns
        (Object): Ocean VNG API response
        """

        response = self.send_get(
            url=self.__base_ocean_vng_url,
            entity_name="ocean_aks_vng",
            query_params=dict(oceanId=ocean_id)
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def delete_ocean_vng(self, vng_id: str):
        """
        Delete an Ocean Cluster

        # Arguments
        vng_id (String): ID of the Ocean VNG

        # Returns
        (Object): Ocean VNG API response
        """
        return self.send_delete(
            url=self.__base_ocean_vng_url + "/" + vng_id,
            entity_name="ocean_aks_vng"
        )

    def import_vng_configuration(self, node_pool_name: str, ocean_id: str):
        """
        Import cluster configuration of an AKS cluster to use in create_ocean_cluster api call 

        # Returns
        (Object): Ocean API response
        """

        response = self.send_post_with_params(
            body=None,
            url=self.__base_ocean_vng_url+"/import",
            entity_name='ocean_aks',
            user_query_params=dict(nodePoolName=node_pool_name, oceanId=ocean_id))

        formatted_response = self.convert_json(response,
                                               self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def launch_new_nodes(self, node_config: azure_ocean.LaunchNewNodes):
        """
        Launch new nodes for a cluster

        # Arguments
        node_config (LaunchNewNodes): LaunchNewNodes object

        # Returns
        (Object): Ocean Launch New Nodes API response
        """

        request = azure_ocean.LaunchNewNodesRequest(node_config)

        excluded_missing_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_ocean_cluster_url+"/launchNewNodes",
            entity_name='ocean_aks')

        formatted_response = self.convert_json(response,
                                               self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_allowed_vng_vm_sizes(self, vng_id: str):
        """
        Get allowed vm sizes for a particular VNG

        # Arguments
        vng_id (String): ID of the Ocean VNG

        # Returns
        (Object): Array of allowed vm sizes list 
        """
        response = self.send_get(
            url=self.__base_ocean_vng_url + "/vmSizes",
            entity_name="ocean_aks_vng",
            query_params=dict(virtualNodeGroupId=vng_id))

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def initiate_roll(self, ocean_id: str, cluster_roll: azure_ocean.Roll):
        """
        Initiate Cluster Rolls

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        cluster_roll (Roll): Cluster Roll / Node Pool names/ VNG Ids

        # Returns
        (Object): Cluster Roll API response
        """
        roll_request = azure_ocean.ClusterRollInitiateRequest(cluster_roll)

        excluded_missing_dict = self.exclude_missing(
            json.loads(roll_request.toJSON()))

        formatted_missing_dict = self.convert_json_with_list_of_lists(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/roll",
            entity_name='ocean (Cluster Roll)')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_roll(self, ocean_id: str, roll_id: str):
        """
        Get status for a roll of an Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        account_id (String): The ID of the account associated with your token.
        roll_id (String): Ocean cluster roll identifier

        # Returns
        (Object): Cluster Roll API response
        """
        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/roll/" + roll_id,
            entity_name="ocean (Cluster Roll)"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def list_rolls(self, ocean_id: str):
        """
        Get status for all rolls of an Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): List of Cluster Roll API response
        """
        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/roll",
            entity_name="ocean (Cluster Roll)"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def stop_roll(self, ocean_id: str, roll_id: str):
        """
        Stop roll of an Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        account_id (String): The ID of the account associated with your token.
        roll_id (String): Ocean cluster roll identifier

        # Returns
        (Object): Cluster Roll API response
        """
        response = self.send_put(
            url=self.__base_ocean_cluster_url + "/" +
            ocean_id + "/roll/" + roll_id + "/stop",
            entity_name="ocean (Cluster Roll)"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def create_migration(self, ocean_id: str, migration: azure_ocean.Migration):
        """
        Create a migration for a given existing instances.

        # Arguments
        migration (Migration): Migration Object

        # Returns
        (Object): Migration create response
        """
        migration = azure_ocean.MigrationRequest(migration)

        excluded_missing_dict = self.exclude_missing(
            json.loads(migration.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/migration",
            entity_name='ocean_azure_migration')

        formatted_response = self.convert_json(response,
                                               self.camel_to_underscore)

        return formatted_response["response"]

    def get_migration_discovery(self, ocean_id: str, should_fetch_pods: bool):
        """
        Get information about nodes which can be migrated into Ocean.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        should_fetch_pods (bool): Should fetch data about running pods for each node.

        # Returns
        (Object): Ocean API response
        """
        query_params = dict(shouldFetchPods=should_fetch_pods)

        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/migration/discovery",
            entity_name="ocean_azure_migration",
            query_params=query_params
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]

    def stop_migration(self, ocean_id: str, migration_id: str):
        """
        Stop an ongoing Workload Migration.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        migration_id (String): The migration identifier of a specific migration

        # Returns
        (Object): Ocean Migration response 
        """

        response = self.send_put(
            url=self.__base_ocean_cluster_url + "/" +
            ocean_id + "/migration/" + migration_id + "/stop",
            entity_name="ocean_azure_migration",
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]

    def get_migration(self, ocean_id: str, migration_id: str):
        """
        Get Migration full info and status for an Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        migration_id (String): The migration identifier of a specific migration.

        # Returns
        (Object): Ocean API response 
        """

        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" +
            ocean_id + "/migration/" + migration_id,
            entity_name="ocean_azure_migration"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]

    def list_migrations(self, ocean_id: str):
        """
        Get summary of migrations history for an Ocean cluster.

        # Returns
        (Object): Ocean Migrations response 
        """

        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/migration",
            entity_name="ocean_azure_migration",
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]
    # endregion



import json
from enum import Enum
from typing import List

none = "d3043820717d74d9a17694c176d39733"


# region Capacity
class Capacity:
    """
    # Arguments
    minimum: int
    maximum: int
    target: int
    unit: str
    """

    def __init__(self,
                 minimum: int = none,
                 maximum: int = none,
                 target: int = none,
                 unit: str = none):
        self.minimum = minimum
        self.maximum = maximum
        self.target = target
        self.unit = unit


# endregion


# region Strategy
class RevertToPreemptible:
    """
    # Arguments
    perform_at: str
    """

    def __init__(self,
                 perform_at: str = none):
        self.perform_at = perform_at


class ProvisioningModel(Enum):
    spot = 'SPOT'
    preemptible = 'PREEMPTIBLE'


class Strategy:
    """
    # Arguments
    preemptible_percentage: int
    on_demand_count: int
    draining_timeout: int
    fallback_to_od: bool
    optimization_windows: List[str]
    provisioning_model: ProvisioningModel
    revert_to_preemptible: RevertToPreemptible
    """

    def __init__(
            self,
            preemptible_percentage: int = none,
            on_demand_count: int = none,
            draining_timeout: int = none,
            fallback_to_od: bool = none,
            optimization_windows: List[str] = none,
            provisioning_model: ProvisioningModel = none,
            revert_to_preemptible: RevertToPreemptible = none):
        self.preemptible_percentage = preemptible_percentage
        self.on_demand_count = on_demand_count
        self.draining_timeout = draining_timeout
        self.fallback_to_od = fallback_to_od
        self.optimization_windows = optimization_windows
        self.provisioning_model = provisioning_model
        self.revert_to_preemptible = revert_to_preemptible


# endregion


# region Scaling
class ScalingPolicyAction:
    """
    # Arguments
    scaling_type: str
    adjustment: int
    """

    def __init__(self,
                 scaling_type: str = none,
                 adjustment: int = none):
        self.type = scaling_type
        self.adjustment = adjustment


class ScalingPolicyDimension:
    """
    # Arguments
    name: str
    value: str
    """

    def __init__(self, name=none, value=none):
        self.name = name
        self.value = value


class ScalingPolicy:
    """
    # Arguments
    action: ScalingPolicyAction
    cooldown: int
    dimensions: List[ScalingPolicyDimension]
    evaluation_periods: int
    metric_name: str
    namespace: str
    operator: str
    period: int
    policy_name: str
    source: str
    statistic: str
    threshold: int
    unit: str
    """

    def __init__(
            self,
            action: ScalingPolicyAction = none,
            cooldown: int = none,
            dimensions: List[ScalingPolicyDimension] = none,
            evaluation_periods: int = none,
            metric_name: str = none,
            namespace: str = none,
            operator: str = none,
            period: int = none,
            policy_name: str = none,
            source: str = none,
            statistic: str = none,
            threshold: int = none,
            unit: str = none):
        self.source = source
        self.policy_name = policy_name
        self.namespace = namespace
        self.metric_name = metric_name
        self.dimensions = dimensions
        self.statistic = statistic
        self.unit = unit
        self.threshold = threshold
        self.period = period
        self.evaluation_periods = evaluation_periods
        self.cooldown = cooldown
        self.action = action
        self.operator = operator


class Scaling:
    """
    # Arguments
    up:  List[ScalingPolicy]
    down: List[ScalingPolicy]
    """

    def __init__(self,
                 up: List[ScalingPolicy] = none,
                 down: List[ScalingPolicy] = none):
        self.up = up
        self.down = down


# endregion


# region ThirdPartiesIntegration
class DockerSwarmConfiguration:
    """
    # Arguments
    master_host: str
    master_port: int
    """

    def __init__(self,
                 master_host: str = none,
                 master_port: int = none):
        self.master_host = master_host
        self.master_port = master_port


class Down:
    """
    # Arguments
    evaluation_periods: int
    """

    def __init__(self, evaluation_periods: int = none):
        self.evaluation_periods = evaluation_periods


class Headroom:
    """
    # Arguments
    cpu_per_unit: int
    memory_per_unit: int
    num_of_units: int
    """

    def __init__(
            self,
            cpu_per_unit: int = none,
            memory_per_unit: int = none,
            num_of_units: int = none):
        self.cpu_per_unit = cpu_per_unit
        self.memory_per_unit = memory_per_unit
        self.num_of_units = num_of_units


class AutoScale:
    """
    # Arguments
    is_enabled: bool
    is_auto_config: bool
    cooldown: int
    headroom: Headroom
    down: Down
    """

    def __init__(
            self,
            is_enabled: bool = none,
            is_auto_config: bool = none,
            cooldown: int = none,
            headroom: Headroom = none,
            down: Down = none):
        self.is_enabled = is_enabled
        self.is_auto_config = is_auto_config
        self.cooldown = cooldown
        self.headroom = headroom
        self.down = down


class GKE:
    """
    # Arguments
    auto_update: bool
    auto_scale: AutoScale
    cluster_identifier: str
    location: str
    """

    def __init__(
            self,
            auto_update: bool = none,
            auto_scale: AutoScale = none,
            cluster_identifier: str = none,
            location: str = none):
        self.auto_update = auto_update
        self.auto_scale = auto_scale
        self.cluster_identifier = cluster_identifier
        self.location = location


class ThirdPartiesIntegration:
    """
    # Arguments
    docker_swarm: DockerSwarmConfiguration
    gke : GKE
    """

    def __init__(
            self,
            docker_swarm: DockerSwarmConfiguration = none,
            gke: GKE = none):
        self.docker_swarm = docker_swarm
        self.gke = gke


# endregion


# region Compute
class Metadata:
    """
    # Arguments
    key: str
    value: str
    """

    def __init__(self, key: str = none, value: str = none):
        self.key = key
        self.value = value


class NamedPorts:
    """
    # Arguments
    name: str
    ports: List[int]
    """

    def __init__(
            self,
            name: str = none,
            ports: List[int] = none):
        self.name = name
        self.ports = ports


class BackendServices:
    """
    # Arguments
    backend_service_name: str
    location_type: str
    scheme: str
    named_ports: NamedPorts
    """

    def __init__(
            self,
            backend_service_name: str = none,
            location_type: str = none,
            scheme: str = none,
            named_ports: NamedPorts = none):
        self.backend_service_name = backend_service_name
        self.location_type = location_type
        self.scheme = scheme
        self.named_ports = named_ports


class BackendServiceConfig:
    """
    # Arguments
    backend_services: List[BackendServices]
    """

    def __init__(self, backend_services: List[BackendServices] = none):
        self.backend_services = backend_services


class InitializeParams:
    """
    # Arguments
    disk_size_gb: int
    disk_type: str
    source_image: str
    """

    def __init__(
            self,
            disk_size_gb: int = none,
            disk_type: str = none,
            source_image: str = none):
        self.disk_size_gb = disk_size_gb
        self.disk_type = disk_type
        self.source_image = source_image


class Disk:
    """
    # Arguments
    auto_delete: bool
    boot: bool
    device_name: str
    initialize_params: InitializeParams
    interface: str
    mode: str
    source: str
    disk_type: str
    """

    def __init__(
            self,
            auto_delete: bool = none,
            boot: bool = none,
            device_name: str = none,
            initialize_params: InitializeParams = none,
            interface: str = none,
            mode: str = none,
            source: str = none,
            disk_type: str = none):
        self.auto_delete = auto_delete
        self.boot = boot
        self.device_name = device_name
        self.initialize_params = initialize_params
        self.interface = interface
        self.mode = mode
        self.source = source
        self.type = disk_type


class NetworkInterface:
    """
    # Arguments
    network: str
    project_id: str
    """

    def __init__(
            self,
            network: str = none,
            project_id: str = none):
        self.network = network
        self.project_id = project_id


class LaunchSpecification:
    """
    # Arguments
    metadata: List[Metadata]
    tags: List[str]
    backend_service_config: BackendServiceConfig
    startup_script: str
    disks: List[Disk]
    network_interfaces: List[NetworkInterface]
    ip_forwarding: bool
    shutdown_script: str
    min_cpu_platform: str
    instance_name_prefix: str
    """

    def __init__(
            self,
            metadata: List[Metadata] = none,
            tags: List[str] = none,
            backend_service_config: BackendServiceConfig = none,
            startup_script: str = none,
            disks: List[Disk] = none,
            network_interfaces: List[NetworkInterface] = none,
            ip_forwarding: bool = none,
            shutdown_script: str = none,
            min_cpu_platform: str = none,
            instance_name_prefix: str = none):
        self.metadata = metadata
        self.tags = tags
        self.backend_service_config = backend_service_config
        self.startup_script = startup_script
        self.disks = disks
        self.network_interfaces = network_interfaces
        self.ip_forwarding = ip_forwarding
        self.shutdown_script = shutdown_script
        self.min_cpu_platform = min_cpu_platform
        self.instance_name_prefix = instance_name_prefix


class CustomInstanceTypes:
    """
    # Arguments
    v_cPU: int
    memory_giB: int
    """

    def __init__(
            self,
            v_cPU: int = none,
            memory_giB: int = none):
        self.v_cPU = v_cPU
        self.memory_giB = memory_giB


class PreferredPreemtible:
    """
    # Arguments
    custom: CustomInstanceTypes
    preemptible: List[str]
    """

    def __init__(self,
                 custom: CustomInstanceTypes = none,
                 preemptible: List[str] = none):
        self.custom = custom
        self.preemptible = preemptible


class InstanceTypes:
    """
    # Arguments
    ondemand: str
    preemptible: List[str]
    custom: CustomInstanceTypes
    preferred: PreferredPreemtible
    """

    def __init__(
            self,
            ondemand: str = none,
            preemptible: List[str] = none,
            custom: CustomInstanceTypes = none,
            preferred: PreferredPreemtible = none):
        self.ondemand = ondemand
        self.preemptible = preemptible
        self.custom = custom
        self.preferred = preferred


class Gpu:
    """
    # Arguments
    gpu_type: str
    count: int
    """

    def __init__(
            self,
            gpu_type: str = none,
            count: int = none):
        self.type = gpu_type
        self.count = count


class Health:
    """
    # Arguments
    grace_period: str
    auto_healing: bool
    health_check_type: str
    unhealthy_duration: int
    """

    def __init__(
            self,
            grace_period: str = none,
            auto_healing: bool = none,
            health_check_type: str = none,
            unhealthy_duration: int = none):
        self.grace_period = grace_period
        self.auto_healing = auto_healing
        self.health_check_type = health_check_type
        self.unhealthy_duration = unhealthy_duration


class Subnet:
    """
    # Arguments
    region: str
    subnet_names: List[str]
    """

    def __init__(
            self,
            region: str = none,
            subnet_names: List[str] = none):
        self.region = region
        self.subnet_names = subnet_names


class Compute:
    """
    # Arguments
    launch_specification: LaunchSpecification
    instance_types: InstanceTypes
    gpu: Gpu
    health: Health
    availability_zones: List[str]
    subnets: List[Subnet]
    preferred_availability_zones: List[str]
    """

    def __init__(
            self,
            launch_specification: LaunchSpecification = none,
            instance_types: InstanceTypes = none,
            gpu: Gpu = none,
            health: Health = none,
            availability_zones: List[str] = none,
            subnets: List[Subnet] = none,
            preferred_availability_zones: List[str] = none):
        self.launch_specification = launch_specification
        self.instance_types = instance_types
        self.gpu = gpu
        self.health = health
        self.availability_zones = availability_zones
        self.subnets = subnets
        self.preferred_availability_zones = preferred_availability_zones


# endregion


# region Scheduling
class Tasks:
    """
    # Arguments
    cron_expression: str
    is_enabled: bool
    max_capacity: int
    min_capacity: int
    target_capacity: int
    task_type: str
    """

    def __init__(self,
                 cron_expression: str = none,
                 is_enabled: bool = none,
                 max_capacity: int = none,
                 min_capacity: int = none,
                 target_capacity: int = none,
                 task_type: str = none):
        self.cron_expression = cron_expression
        self.is_enabled = is_enabled
        self.max_capacity = max_capacity
        self.min_capacity = min_capacity
        self.target_capacity = target_capacity
        self.task_type = task_type


class Scheduling:
    """
    # Arguments
    tasks: List[Tasks]
    """

    def __init__(self,
                 tasks: List[Tasks] = none):
        self.tasks = tasks
# endregion


# region Elastigroup
class Elastigroup:
    """
    # Arguments
    name: str
    description: str
    capacity: Capacity
    strategy: Strategy
    scaling: Scaling
    third_parties_integration: ThirdPartiesIntegration
    compute: Compute
    scheduling: Scheduling
    """

    def __init__(
            self,
            name: str = none,
            description: str = none,
            capacity: Capacity = none,
            strategy: Strategy = none,
            scaling: Scaling = none,
            third_parties_integration: ThirdPartiesIntegration = none,
            compute: Compute = none,
            scheduling: Scheduling = none):
        self.name = name
        self.description = description
        self.capacity = capacity
        self.strategy = strategy
        self.scaling = scaling
        self.third_parties_integration = third_parties_integration
        self.compute = compute
        self.scheduling = scheduling


class ElastigroupCreationRequest:
    def __init__(self, elastigroup: Elastigroup):
        self.group = elastigroup

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ElastigroupUpdateRequest:
    def __init__(self, elastigroup: Elastigroup):
        self.group = elastigroup

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


# endregion


# region Deployment
class RollGroup:
    """
    # Arguments
    batch_size_percentage: int
    grace_period: int
    """

    def __init__(
            self,
            batch_size_percentage: int = none,
            grace_period: int = none):
        self.batch_size_percentage = batch_size_percentage
        self.grace_period = grace_period


class ElastigroupRollRequest:
    def __init__(self, group_roll: RollGroup):
        self.batch_size_percentage = group_roll.batch_size_percentage
        self.grace_period = group_roll.grace_period

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion


# region Detach Instance
class DetachConfiguration:
    """
    # Arguments
    instances_to_detach: List[str]
    should_terminate_instances: bool
    draining_timeout: int
    should_decrement_target_capacity: bool
    """

    def __init__(
            self,
            instances_to_detach: List[str] = none,
            should_terminate_instances: bool = none,
            draining_timeout: int = none,
            should_decrement_target_capacity: bool = none):
        self.instances_to_detach = instances_to_detach
        self.should_terminate_instances = should_terminate_instances
        self.draining_timeout = draining_timeout
        self.should_decrement_target_capacity = should_decrement_target_capacity


class ElastigroupDetachInstancesRequest:
    def __init__(self, detach_configuration: DetachConfiguration):
        self.should_decrement_target_capacity = detach_configuration.should_decrement_target_capacity
        self.draining_timeout = detach_configuration.draining_timeout
        self.instances_to_detach = detach_configuration.instances_to_detach
        self.should_terminate_instances = detach_configuration.should_terminate_instances

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)
# endregion



import json

none = "d3043820717d74d9a17694c176d39733"

# region ASG


class ASG:
    """
    # Arguments
    product: str
    spot_instance_types: List[str]
    name: str
    """

    def __init__(
            self,
            product=none,
            spot_instance_types=none,
            name=none):

        self.product = product
        self.spot_instance_types = spot_instance_types
        self.name = name

# endregion


class ImportASGRequest:
    def __init__(self, group):
        self.group = group

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
